/*!
* Copyright Adaptavist 2022 (c) All rights reserved
*/
import { FormData, Blob, Request, Response, BaseApiCore, constructUrl, executeHttpCall } from '@managed-api/commons-core';
import { constructHeaders, errorMessageBuilder } from './common';
import { customErrorHandlers, errorStrategyBuilderConstructor } from './errorStrategy';
export class JiraOnPremApiCore extends BaseApiCore {
    constructor(options) {
        super();
        this.options = options;
        if (!this.options?.errorStrategy && this.options?.errorStrategy !== null) {
            this.setGlobalErrorStrategy((builder) => builder.retryOnRateLimiting(-1));
        }
        this.Issue = new IssueGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Application = new ApplicationGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Workflow = new WorkflowGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Dashboard = new DashboardGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Filter = new FilterGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Group = new GroupGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Instance = new InstanceGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Myself = new MyselfGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Permission = new PermissionGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Project = new ProjectGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Screen = new ScreenGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.User = new UserGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Role = new RoleGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.All = new AllGroup(this.Issue, this.Application, this.Workflow, this.Dashboard, this.Filter, this.Group, this.Instance, this.Myself, this.Permission, this.Project, this.Screen, this.User, this.Role);
    }
    setGlobalErrorStrategy(errorStrategy) {
        this.options = {
            ...this.options ?? {},
            errorStrategy
        };
    }
    buildResponse(url, status, statusText, headers, body) {
        const response = new Response(url, status, statusText, headers, body);
        response.setBufferImplementation(this.implementation);
        return response;
    }
    getGlobalErrorStrategy() {
        return this.options?.errorStrategy;
    }
}
class AllGroup {
    constructor(Issue, Application, Workflow, Dashboard, Filter, Group, Instance, Myself, Permission, Project, Screen, User, Role) {
        this.Issue = Issue;
        this.Application = Application;
        this.Workflow = Workflow;
        this.Dashboard = Dashboard;
        this.Filter = Filter;
        this.Group = Group;
        this.Instance = Instance;
        this.Myself = Myself;
        this.Permission = Permission;
        this.Project = Project;
        this.Screen = Screen;
        this.User = User;
        this.Role = Role;
    }
    /**
    * Get all the ProjectRoles available in JIRA. Currently this list is global.
    * Alternative usage: Role.Project.getRoles
    * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRoles
    * @param options Request options.
    */
    async getGlobalProjectRoles(options) {
        return this.Role.Project.getRoles(options);
    }
    /**
     * Creates a new ProjectRole to be available in Jira. The created role does not have any default actors assigned.
     * Alternative usage: Role.Project.createRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-createProjectRole
     * @param options Request options.
     */
    async createGlobalProjectRole(options) {
        return this.Role.Project.createRole(options);
    }
    /**
     * Get a specific ProjectRole available in JIRA.
     * Alternative usage: Role.Project.getRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRolesById
     * @param options Request options.
     */
    async getGlobalProjectRole(options) {
        return this.Role.Project.getRole(options);
    }
    /**
     * Partially updates a roles name or description.
     * Alternative usage: Role.Project.updateRolePartially
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-partialUpdateProjectRole
     * @param options Request options.
     */
    async updateGlobalProjectRolePartially(options) {
        return this.Role.Project.updateRolePartially(options);
    }
    /**
     * Fully updates a roles. Both name and description must be given.
     * Alternative usage: Role.Project.updateRoleFully
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRolesById
     * @param options Request options.
     */
    async updateGlobalProjectRoleFully(options) {
        return this.Role.Project.updateRoleFully(options);
    }
    /**
     * Deletes a role. May return 403 in the future.
     * Alternative usage: Role.Project.deleteRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-deleteProjectRole
     * @param options Request options.
     */
    async deleteGlobalProjectRole(options) {
        return this.Role.Project.deleteRole(options);
    }
    /**
     * Gets default actors for the given role.
     * Alternative usage: Role.Project.Actor.getActors
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-getProjectRoleActorsForRole
     * @param options Request options.
     */
    async getGlobalProjectRoleActors(options) {
        return this.Role.Project.Actor.getActors(options);
    }
    /**
     * Adds default actors to the given role. The request data should contain a list of usernames or a list of groups to add.
     * Alternative usage: Role.Project.Actor.addActors
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-addProjectRoleActorsToRole
     * @param options Request options.
     */
    async addGlobalProjectRoleActors(options) {
        return this.Role.Project.Actor.addActors(options);
    }
    /**
     * Removes default actor from the given role.
     * Alternative usage: Role.Project.Actor.deleteActors
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-deleteProjectRoleActorsFromRole
     * @param options Request options.
     */
    async deleteGlobalProjectRoleActors(options) {
        return this.Role.Project.Actor.deleteActors(options);
    }
    /**
     * Returns a user.
     * Alternative usage: User.getUser
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-getUser
     * @param options Request options.
     */
    async getUser(options) {
        return this.User.getUser(options);
    }
    /**
     * Modify user. The "value" fields present will override the existing value. Fields skipped in request will not be changed.
     * Alternative usage: User.updateUser
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-updateUser
     * @param options Request options.
     */
    async updateUser(options) {
        return this.User.updateUser(options);
    }
    /**
     * Create user. By default created user will not be notified with email. If password field is not set then password will be randomly generated.
     * Alternative usage: User.createUser
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-createUser
     * @param options Request options.
     */
    async createUser(options) {
        return this.User.createUser(options);
    }
    /**
     * Removes user and its references (like project roles associations, watches, history).
     * Alternative usage: User.removeUser
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-removeUser
     * @param options Request options.
     */
    async removeUser(options) {
        return this.User.removeUser(options);
    }
    /**
     * Add user to given application. Admin permission will be required to perform this operation.
     * Alternative usage: User.addUserToApplication
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-addUserToApplication
     * @param options Request options.
     */
    async addUserToApplication(options) {
        return this.User.addUserToApplication(options);
    }
    /**
     * Remove user from given application. Admin permission will be required to perform this operation.
     * Alternative usage: User.removeUserFromApplication
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-removeUserFromApplication
     * @param options Request options.
     */
    async removeUserFromApplication(options) {
        return this.User.removeUserFromApplication(options);
    }
    /**
     * Modify user password.
     * Alternative usage: User.changeUserPassword
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-changeUserPassword
     * @param options Request options.
     */
    async changeUserPassword(options) {
        return this.User.changeUserPassword(options);
    }
    /**
     * Returns a list of users that match the search string. This resource cannot be accessed anonymously. Please note that this resource should be called with an issue key when a list of assignable users is retrieved for editing. For create only a project key should be supplied. The list of assignable users may be incorrect if it's called with the project key for editing.
     * Alternative usage: User.Search.findAssignableUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findAssignableUsers
     * @param options Request options.
     */
    async findAssignableUsers(options) {
        return this.User.Search.findAssignableUsers(options);
    }
    /**
     * Returns a list of active users that match the search string and have all specified permissions for the project or issue.
     * This resource can be accessed by users with ADMINISTER_PROJECT permission for the project or global ADMIN or SYSADMIN rights.
     * Alternative usage: User.Search.findUsersWithAllPermissions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsersWithAllPermissions
     * @param options Request options.
     */
    async findUsersWithAllPermissions(options) {
        return this.User.Search.findUsersWithAllPermissions(options);
    }
    /**
     * Returns a list of users that match the search string. This resource cannot be accessed anonymously.
     * Alternative usage: User.Search.findUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsers
     * @param options Request options.
     */
    async findUsers(options) {
        return this.User.Search.findUsers(options);
    }
    /**
     * Returns a list of active users that match the search string. This resource cannot be accessed anonymously and requires the Browse Users global permission. Given an issue key this resource will provide a list of users that match the search string and have the browse issue permission for the issue provided.
     * Alternative usage: User.Search.findUsersWithBrowsePermission
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsersWithBrowsePermission
     * @param options Request options.
     */
    async findUsersWithBrowsePermission(options) {
        return this.User.Search.findUsersWithBrowsePermission(options);
    }
    /**
     * Gets available fields for screen. i.e ones that haven't already been added.
     * Alternative usage: Screen.getFieldsToAdd
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getFieldsToAdd
     * @param options Request options.
     */
    async getScreenFieldsToAdd(options) {
        return this.Screen.getFieldsToAdd(options);
    }
    /**
     * Gets available field screens.
     * Alternative usage: Screen.getScreens
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllScreens
     * @param options Request options.
     */
    async getScreens(options) {
        return this.Screen.getScreens(options);
    }
    /**
     * Creates tab for given screen.
     * Alternative usage: Screen.Tab.addTab
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addTab
     * @param options Request options.
     */
    async addScreenTab(options) {
        return this.Screen.Tab.addTab(options);
    }
    /**
     * Returns a list of all tabs for the given screen.
     * Alternative usage: Screen.Tab.getTabs
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllTabs
     * @param options Request options.
     */
    async getScreenTabs(options) {
        return this.Screen.Tab.getTabs(options);
    }
    /**
     * Renames tab on given screen.
     * Alternative usage: Screen.Tab.renameTab
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-renameTab
     * @param options Request options.
     */
    async renameScreenTab(options) {
        return this.Screen.Tab.renameTab(options);
    }
    /**
     * Deletes tab to give screen.
     * Alternative usage: Screen.Tab.deleteTab
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-deleteTab
     * @param options Request options.
     */
    async deleteScreenTab(options) {
        return this.Screen.Tab.deleteTab(options);
    }
    /**
     * Moves tab position.
     * Alternative usage: Screen.Tab.moveTab
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-moveTab
     * @param options Request options.
     */
    async moveScreenTab(options) {
        return this.Screen.Tab.moveTab(options);
    }
    /**
     * Gets all fields for a given tab.
     * Alternative usage: Screen.Field.getFields
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllFields
     * @param options Request options.
     */
    async getScreenFields(options) {
        return this.Screen.Field.getFields(options);
    }
    /**
     * Adds field to the given tab.
     * Alternative usage: Screen.Field.addField
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addField
     * @param options Request options.
     */
    async addScreenField(options) {
        return this.Screen.Field.addField(options);
    }
    /**
     * Removes field from given tab.
     * Alternative usage: Screen.Field.removeField
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-removeField
     * @param options Request options.
     */
    async removeScreenField(options) {
        return this.Screen.Field.removeField(options);
    }
    /**
     * Moves field on the given tab.
     * Alternative usage: Screen.Field.moveField
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-moveField
     * @param options Request options.
     */
    async moveScreenField(options) {
        return this.Screen.Field.moveField(options);
    }
    /**
     * Adds field or custom field to the default tab.
     * Alternative usage: Screen.Field.addToDefaultScreen
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addFieldToDefaultScreen
     * @param options Request options.
     */
    async addFieldToDefaultScreen(options) {
        return this.Screen.Field.addToDefaultScreen(options);
    }
    /**
     * Returns all projects which are visible for the currently logged in user. If no user is logged in, it returns the list of projects that are visible when using anonymous access.
     * Alternative usage: Project.getProjects
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getAllProjects
     * @param options Request options.
     */
    async getProjects(options) {
        return this.Project.getProjects(options);
    }
    /**
     * Creates a new project.
     * Alternative usage: Project.createProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-createProject
     * @param options Request options.
     */
    async createProject(options) {
        return this.Project.createProject(options);
    }
    /**
     * Contains a full representation of a project in JSON format.
     * Alternative usage: Project.getProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProject
     * @param options Request options.
     */
    async getProject(options) {
        return this.Project.getProject(options);
    }
    /**
     * Updates a project. Only non null values sent in JSON will be updated in the project.
     * Alternative usage: Project.updateProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-updateProject
     * @param options Request options.
     */
    async updateProject(options) {
        return this.Project.updateProject(options);
    }
    /**
     * Deletes a project.
     * Alternative usage: Project.deleteProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-deleteProject
     * @param options Request options.
     */
    async deleteProject(options) {
        return this.Project.deleteProject(options);
    }
    /**
     * Archives a project. For DC only.
     * Alternative usage: Project.archiveProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-archiveProject
     * @param options Request options.
     */
    async archiveProject(options) {
        return this.Project.archiveProject(options);
    }
    /**
     * Contains a full representation of a the specified project's components.
     * Alternative usage: Project.getComponents
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectComponents
     * @param options Request options.
     */
    async getProjectComponents(options) {
        return this.Project.getComponents(options);
    }
    /**
     * Restore an archived project. In case of success restored project should be re-indexed. For DC only.
     * Alternative usage: Project.restoreProject
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-restoreProject
     * @param options Request options.
     */
    async restoreProject(options) {
        return this.Project.restoreProject(options);
    }
    /**
     * Get all issue types with valid status values for a project.
     * Alternative usage: Project.getStatuses
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getAllStatuses
     * @param options Request options.
     */
    async getProjectStatuses(options) {
        return this.Project.getStatuses(options);
    }
    /**
     * Updates the type of a project.
     * Alternative usage: Project.updateProjectType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-updateProjectType
     * @param options Request options.
     */
    async updateProjectType(options) {
        return this.Project.updateProjectType(options);
    }
    /**
     * Returns all versions for the specified project. Results are paginated.
     * Alternative usage: Project.getVersionsPaginated
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectVersionsPaginated
     * @param options Request options.
     */
    async getProjectVersionsPaginated(options) {
        return this.Project.getVersionsPaginated(options);
    }
    /**
     * Contains a full representation of a the specified project's versions.
     * Alternative usage: Project.getVersions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectVersions
     * @param options Request options.
     */
    async getProjectVersions(options) {
        return this.Project.getVersions(options);
    }
    /**
     * Gets a full representation of a priority scheme in JSON format used by specified project.
     * Alternative usage: Project.Priority.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/priorityscheme-getAssignedPriorityScheme
     * @param options Request options.
     */
    async getPriorityScheme(options) {
        return this.Project.Priority.Scheme.getScheme(options);
    }
    /**
     * Resource for associating permission schemes and projects.
     * Alternative usage: Project.Permission.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/permissionscheme-getAssignedPermissionScheme
     * @param options Request options.
     */
    async getProjectPermissionScheme(options) {
        return this.Project.Permission.Scheme.getScheme(options);
    }
    /**
     * Gets a notification scheme associated with the project.
     * Alternative usage: Project.Notification.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/notificationscheme-getNotificationScheme
     * @param options Request options.
     */
    async getProjectNotificationScheme(options) {
        return this.Project.Notification.Scheme.getScheme(options);
    }
    /**
     * Returns the issue security scheme for project.
     * Alternative usage: Project.Issue.Security.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/issuesecuritylevelscheme-getIssueSecurityScheme
     * @param options Request options.
     */
    async getProjectIssueSecurityScheme(options) {
        return this.Project.Issue.Security.Scheme.getScheme(options);
    }
    /**
     * Create a version via POST.
     * Alternative usage: Project.Version.createVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-createVersion
     * @param options Request options.
     */
    async createProjectVersion(options) {
        return this.Project.Version.createVersion(options);
    }
    /**
     * Modify a version's sequence within a project.
     * Alternative usage: Project.Version.moveVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-moveVersion
     * @param options Request options.
     */
    async moveProjectVersion(options) {
        return this.Project.Version.moveVersion(options);
    }
    /**
     * Returns a project version.
     * Alternative usage: Project.Version.getVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersion
     * @param options Request options.
     */
    async getProjectVersion(options) {
        return this.Project.Version.getVersion(options);
    }
    /**
     * Modify a version via PUT. Any fields present in the PUT will override existing values. As a convenience, if a field is not present, it is silently ignored.
     * Alternative usage: Project.Version.updateVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-updateVersion
     * @param options Request options.
     */
    async updateProjectVersion(options) {
        return this.Project.Version.updateVersion(options);
    }
    /**
     * Delete a project version.
     * Alternative usage: Project.Version.deleteVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-delete
     * @param options Request options.
     */
    async deleteProjectVersion(options) {
        return this.Project.Version.deleteVersion(options);
    }
    /**
     * Merge versions.
     * Alternative usage: Project.Version.mergeVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-merge
     * @param options Request options.
     */
    async mergeProjectVersion(options) {
        return this.Project.Version.mergeVersion(options);
    }
    /**
     * Delete a project version.
     * Alternative usage: Project.Version.removeAndReplaceVersion
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-delete
     * @param options Request options.
     */
    async removeAndReplaceProjectVersion(options) {
        return this.Project.Version.removeAndReplaceVersion(options);
    }
    /**
     * Returns a bean containing the number of fixed in and affected issues for the given version.
     * Alternative usage: Project.Version.Count.getVersionRelatedIssueCount
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersionRelatedIssues
     * @param options Request options.
     */
    async getProjectVersionRelatedIssueCount(options) {
        return this.Project.Version.Count.getVersionRelatedIssueCount(options);
    }
    /**
     * Returns the number of unresolved issues for the given version
     * Alternative usage: Project.Version.Count.getVersionUnresolvedIssueCount
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersionUnresolvedIssues
     * @param options Request options.
     */
    async getProjectVersionUnresolvedIssueCount(options) {
        return this.Project.Version.Count.getVersionUnresolvedIssueCount(options);
    }
    /**
     * Returns the remote version links associated with the given version ID.
     * Alternative usage: Project.Version.Remote.getLinksById
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-getRemoteVersionLinksByVersionId
     * @param options Request options.
     */
    async getRemoteVersionLinksById(options) {
        return this.Project.Version.Remote.getLinksById(options);
    }
    /**
     * Create a remote version link via POST.
     * Alternative usage: Project.Version.Remote.createOrUpdateLinkById
     * The link's global ID will be taken from the JSON payload if provided; otherwise, it will be generated.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-createOrUpdateRemoteVersionLink
     * @param options Request options.
     */
    async createOrUpdateRemoteVersionLinkById(options) {
        return this.Project.Version.Remote.createOrUpdateLinkById(options);
    }
    /**
     * Delete all remote version links for a given version ID.
     * Alternative usage: Project.Version.Remote.deleteLinksById
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-deleteRemoteVersionLinksByVersionId
     * @param options Request options.
     */
    async deleteRemoteVersionLinksById(options) {
        return this.Project.Version.Remote.deleteLinksById(options);
    }
    /**
     * A REST sub-resource representing a remote version link.
     * Alternative usage: Project.Version.Remote.getLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-getRemoteVersionLink
     * @param options Request options.
     */
    async getRemoteVersionLink(options) {
        return this.Project.Version.Remote.getLink(options);
    }
    /**
     * Create a remote version link via POST.
     * Alternative usage: Project.Version.Remote.createOrUpdateLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-createOrUpdateRemoteVersionLink
     * @param options Request options.
     */
    async createOrUpdateRemoteVersionLink(options) {
        return this.Project.Version.Remote.createOrUpdateLink(options);
    }
    /**
     * Delete a specific remote version link with the given version ID and global ID.
     * Alternative usage: Project.Version.Remote.deleteLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-deleteRemoteVersionLink
     * @param options Request options.
     */
    async deleteRemoteVersionLink(options) {
        return this.Project.Version.Remote.deleteLink(options);
    }
    /**
     * Delete a specific remote version link with the given version ID and global ID.
     * Alternative usage: Project.Version.Remote.deleteLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-deleteRemoteVersionLink
     * @param options Request options.
     */
    async getRemoteVersionLinks(options) {
        return this.Project.Version.Remote.getLinks(options);
    }
    /**
     * Validates a project key.
     * Alternative usage: Project.Validation.validateProjectKey
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#projectvalidate-getProject
     * @param options Request options.
     */
    async validateProjectKey(options) {
        return this.Project.Validation.validateProjectKey(options);
    }
    /**
     * Returns all the project types defined on the JIRA instance, not taking into account whether the license to use those project types is valid or not.
     * Alternative usage: Project.Type.getTypes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getAllProjectTypes
     * @param options Request options.
     */
    async getProjectTypes(options) {
        return this.Project.Type.getTypes(options);
    }
    /**
    * Returns the project type with the given key.
    * Alternative usage: Project.Type.getType
    * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getProjectTypeByKey
    * @param options Request options.
    */
    async getProjectType(options) {
        return this.Project.Type.getType(options);
    }
    /**
     * Returns the project type with the given key, if it is accessible to the logged in user. This takes into account whether the user is licensed on the Application that defines the project type.
     * Alternative usage: Project.Type.getAccessibleType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getAccessibleProjectTypeByKey
     * @param options Request options.
     */
    async getAccessibleProjectType(options) {
        return this.Project.Type.getAccessibleType(options);
    }
    /**
     * Returns all security levels for the project that the current logged in user has access to.
     * Alternative usage: Project.Security.getLevels
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/securitylevel-getSecurityLevelsForProject
     * @param options Request options.
     */
    async getProjectSecurityLevels(options) {
        return this.Project.Security.getLevels(options);
    }
    /**
     * Returns all roles in the given project Id or key, with links to full details on each role.
     * Alternative usage: Project.Role.getRoles
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-getProjectRoles
     * @param options Request options.
     */
    async getProjectRoles(options) {
        return this.Project.Role.getRoles(options);
    }
    /**
     * Returns the details for a given project role in a project.
     * Alternative usage: Project.Role.getRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-getProjectRole
     * @param options Request options.
     */
    async getProjectRole(options) {
        return this.Project.Role.getRole(options);
    }
    /**
     * Updates a project role to include the specified actors (users or groups).
     * Alternative usage: Project.Role.Actor.setActors
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-setActors
     * @param options Request options.
     */
    async setProjectRoleActors(options) {
        return this.Project.Role.Actor.setActors(options);
    }
    /**
     * Adds an actor (user or group) to a project role.
     * Alternative usage: Project.Role.Actor.addUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-addActorUsers
     * @param options Request options.
     */
    async addProjectRoleActorUsers(options) {
        return this.Project.Role.Actor.addUsers(options);
    }
    /**
     * Deletes actors (users or groups) from a project role.
     * Delete a user from the role: /rest/api/2/project/{projectIdOrKey}/role/{roleId}?user={username}
     * Delete a group from the role: /rest/api/2/project/{projectIdOrKey}/role/{roleId}?group={groupname}
     * Alternative usage: Project.Role.Actor.deleteActor
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-deleteActor
     * @param options Request options.
     */
    async deleteProjectRoleActor(options) {
        return this.Project.Role.Actor.deleteActor(options);
    }
    /**
     * Create a component via POST.
     * Alternative usage: Project.Component.createComponent
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-createComponent
     * @param options Request options.
     */
    async createProjectComponent(options) {
        return this.Project.Component.createComponent(options);
    }
    /**
     * Returns a project component.
     * Alternative usage: Project.Component.getComponent
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-getComponent
     * @param options Request options.
     */
    async getProjectComponent(options) {
        return this.Project.Component.getComponent(options);
    }
    /**
     * Modify a component via PUT. If leadUserName is an empty string ("") the component lead will be removed.
     * Alternative usage: Project.Component.updateComponent
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-updateComponent
     * @param options Request options.
     */
    async updateProjectComponent(options) {
        return this.Project.Component.updateComponent(options);
    }
    /**
     * Delete a project component.
     * Alternative usage: Project.Component.deleteComponent
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-delete
     * @param options Request options.
     */
    async deleteProjectComponent(options) {
        return this.Project.Component.deleteComponent(options);
    }
    /**
     * Returns counts of issues related to this component.
     * Alternative usage: Project.Component.getRelatedIssueCount
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-getComponentRelatedIssues
     * @param options Request options.
     */
    async getProjectComponentRelatedIssueCount(options) {
        return this.Project.Component.getRelatedIssueCount(options);
    }
    /**
     * Returns all permissions in the system and whether the currently logged in user has them.
     * You can optionally provide a specific context to get permissions for (projectKey OR projectId OR issueKey OR issueId).
     * Alternative usage: Permission.getMyPermissions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#-getPermissions
     * @param options Request options.
     */
    async getMyPermissions(options) {
        return this.Permission.getMyPermissions(options);
    }
    /**
     * Returns all permissions that are present in the JIRA instance - Global, Project and the global ones added by plugins
     * Alternative usage: Permission.getPermissions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#-getAllPermissions
     * @param options Request options.
     */
    async getPermissions(options) {
        return this.Permission.getPermissions(options);
    }
    /**
     * Returns a permission scheme identified by the given id.
     * Alternative usage: Permission.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionScheme
     * @param options Request options.
     */
    async getPermissionSchemes(options) {
        return this.Permission.Scheme.getSchemes(options);
    }
    /**
     * Create a new permission scheme. This method can create schemes with a defined permission set, or without.
     * Alternative usage: Permission.Scheme.createScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-createPermissionScheme
     * @param options Request options.
     */
    async createPermissionScheme(options) {
        return this.Permission.Scheme.createScheme(options);
    }
    /**
     * Returns a permission scheme identified by the given id.
     * Alternative usage: Permission.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionScheme
     * @param options Request options.
     */
    async getPermissionScheme(options) {
        return this.Permission.Scheme.getScheme(options);
    }
    /**
     * Deletes a permission scheme identified by the given id.
     * Alternative usage: Permission.Scheme.deleteScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-deletePermissionScheme
     * @param options Request options.
     */
    async deletePermissionScheme(options) {
        return this.Permission.Scheme.deleteScheme(options);
    }
    /**
     * Updates a permission scheme.
     * If the permissions list is present then it will be set in the permission scheme, which basically means it will overwrite any permission grants that existed in the permission scheme.
     * Sending an empty list will remove all permission grants from the permission scheme.
     * To update just the name and description, do not send permissions list at all.
     * Alternative usage: Permission.Scheme.updateScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-updatePermissionScheme
     * @param options Request options.
     */
    async updatePermissionScheme(options) {
        return this.Permission.Scheme.updateScheme(options);
    }
    /**
     * Returns scheme attribute.
     * Alternative usage: Permission.Scheme.Attribute.getAttribute
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getSchemeAttribute
     * @param options Request options.
     */
    async getPermissionSchemeAttribute(options) {
        return this.Permission.Scheme.Attribute.getAttribute(options);
    }
    /**
     * Returns all permission grants of the given permission scheme.
     * Alternative usage: Permission.Scheme.Grant.getGrants
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemeGrants
     * @param options Request options.
     */
    async getPermissionSchemeGrants(options) {
        return this.Permission.Scheme.Grant.getGrants(options);
    }
    /**
     * Creates a permission grant in a permission scheme.
     * Alternative usage: Permission.Scheme.Grant.createGrant
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-createPermissionGrant
     * @param options Request options.
     */
    async createPermissionSchemeGrant(options) {
        return this.Permission.Scheme.Grant.createGrant(options);
    }
    /**
     * Deletes a permission grant from a permission scheme.
     * Alternative usage: Permission.Scheme.Grant.deleteGrant
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-deletePermissionSchemeEntity
     * @param options Request options.
     */
    async deletePermissionSchemeGrant(options) {
        return this.Permission.Scheme.Grant.deleteGrant(options);
    }
    /**
     * Returns a permission grant identified by the given id.
     * Alternative usage: Permission.Scheme.Grant.getGrant
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemeGrant
     * @param options Request options.
     */
    async getPermissionSchemeGrant(options) {
        return this.Permission.Scheme.Grant.getGrant(options);
    }
    /**
     * Sets preference of the currently logged in user.
     * Preference key must be provided as input parameters (key).
     * Value must be provided as post body. If key or value parameter is not provided - status code 404.
     * If preference is set - status code 204.
     * Alternative usage: Issue.Myself.setPreference
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-setPreference
     * @param options Request options.
     */
    async setMyPreference(options) {
        return this.Myself.setPreference(options);
    }
    /**
     * Removes preference of the currently logged in user. Preference key must be provided as input parameters (key).
     * If key parameter is not provided or wrong - status code 404.
     * If preference is unset - status code 204.
     * Alternative usage: Issue.Myself.removePreference
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-removePreference
     * @param options Request options.
     */
    async removeMyPreference(options) {
        return this.Myself.removePreference(options);
    }
    /**
     * Returns preference of the currently logged in user.
     * Preference key must be provided as input parameter (key).
     * The value is returned exactly as it is. If key parameter is not provided or wrong - status code 404.
     * If value is found - status code 200.
     * Alternative usage: Issue.Myself.getPreference
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-getPreference
     * @param options Request options.
     */
    async getMyPreference(options) {
        return this.Myself.getPreference(options);
    }
    /**
     * Returns currently logged in user.
     * Alternative usage: Myself.getCurrentUser
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#myself-getUser
     * @param options Request options.
     */
    async getCurrentUser(options) {
        return this.Myself.getCurrentUser(options);
    }
    /**
     * Returns a paginated list of notification schemes.
     * Alternative usage: Issue.Notification.Scheme.getSchemes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#notificationscheme-getNotificationSchemes
     * @param options Request options.
     */
    async getNotificationSchemes(options) {
        return this.Issue.Notification.Scheme.getSchemes(options);
    }
    /**
     * Returns a full representation of the notification scheme for the given id.
     * Alternative usage: Issue.Notification.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#notificationscheme-getNotificationScheme
     * @param options Request options.
     */
    async getNotificationScheme(options) {
        return this.Issue.Notification.Scheme.getScheme(options);
    }
    /**
     * Returns all issue security schemes that are defined.
     * Alternative usage: Issue.Security.Scheme.getSchemes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuesecurityschemes-getIssueSecuritySchemes
     * @param options Request options.
     */
    async getIssueSecuritySchemes(options) {
        return this.Issue.Security.Scheme.getSchemes(options);
    }
    /**
     * Returns the issue security scheme along with that are defined.
     * Alternative usage: Issue.Security.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuesecurityschemes-getIssueSecurityScheme
     * @param options Request options.
     */
    async getIssueSecurityScheme(options) {
        return this.Issue.Security.Scheme.getScheme(options);
    }
    /**
     * Returns all work logs for an issue. Work logs won't be returned if the Log work field is hidden for the project.
     * Alternative usage: Issue.Worklog.getWorklogs
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssueWorklog
     * @param options Request options.
     */
    async getIssueWorklogs(options) {
        return this.Issue.Worklog.getWorklogs(options);
    }
    /**
     * Adds a new worklog entry to an issue.
     * Alternative usage: Issue.Worklog.addWorklog
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addWorklog
     * @param options Request options.
     */
    async addIssueWorklog(options) {
        return this.Issue.Worklog.addWorklog(options);
    }
    /**
     * Returns a specific worklog. The work log won't be returned if the Log work field is hidden for the project.
     * Alternative usage: Issue.Worklog.getWorklog
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getWorklog
     * @param options Request options.
     */
    async getIssueWorklog(options) {
        return this.Issue.Worklog.getWorklog(options);
    }
    /**
     * Updates an existing worklog entry. Fields possible for editing are: comment, visibility, started, timeSpent and timeSpentSeconds. Either timeSpent or timeSpentSeconds can be set. Fields which are not set will not be updated. For a request to be valid, it has to have at least one field change.
     * Alternative usage: Issue.Worklog.updateWorklog
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateWorklog
     * @param options Request options.
     */
    async updateIssueWorklog(options) {
        return this.Issue.Worklog.updateWorklog(options);
    }
    /**
     * Deletes an existing worklog entry.
     * Alternative usage: Issue.Worklog.deleteWorklog
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteWorklog
     * @param options Request options.
     */
    async deleteIssueWorklog(options) {
        return this.Issue.Worklog.deleteWorklog(options);
    }
    /**
     * Returns worklogs id and delete time of worklogs that was deleted since given time. The returns set of worklogs is limited to 1000 elements. This API will not return worklogs deleted during last minute.
     * Alternative usage: Issue.Worklog.getDeletedWorklogIds
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getIdsOfWorklogsDeletedSince
     * @param options Request options.
     */
    async getDeletedIssueWorklogIds(options) {
        return this.Issue.Worklog.getDeletedWorklogIds(options);
    }
    /**
     * Returns worklogs for given worklog ids. Only worklogs to which the calling user has permissions, will be included in the result. The returns set of worklogs is limited to 1000 elements.
     * Alternative usage: Issue.Worklog.getWorklogsForIds
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getWorklogsForIds
     * @param options Request options.
     */
    async getIssueWorklogsForIds(options) {
        return this.Issue.Worklog.getWorklogsForIds(options);
    }
    /**
     * Returns worklogs id and update time of worklogs that was updated since given time. The returns set of worklogs is limited to 1000 elements. This API will not return worklogs updated during last minute.
     * Alternative usage: Issue.Worklog.getUpdatedWorklogIds
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getIdsOfWorklogsModifiedSince
     * @param options Request options.
     */
    async getUpdatedIssueWorklogIds(options) {
        return this.Issue.Worklog.getUpdatedWorklogIds(options);
    }
    /**
     * Returns the list of watchers for the issue with the given key.
     * Alternative usage: Issue.Watcher.getWatchers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssueWatchers
     * @param options Request options.
     */
    async getIssueWatchers(options) {
        return this.Issue.Watcher.getWatchers(options);
    }
    /**
     * Adds a user to an issue's watcher list.
     * Alternative usage: Issue.Watcher.addWatcher
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addWatcher
     * @param options Request options.
     */
    async addIssueWatcher(options) {
        return this.Issue.Watcher.addWatcher(options);
    }
    /**
     * Removes a user from an issue's watcher list.
     * Alternative usage: Issue.Watcher.removeWatcher
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-removeWatcher
     * @param options Request options.
     */
    async removeIssueWatcher(options) {
        return this.Issue.Watcher.removeWatcher(options);
    }
    /**
     * Remove your vote from an issue. (i.e. "unvote").
     * Alternative usage: Issue.Vote.removeVote
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-removeVote
     * @param options Request options.
     */
    async removeIssueVote(options) {
        return this.Issue.Vote.addVote(options);
    }
    /**
     * Cast your vote in favour of an issue.
     * Alternative usage: Issue.Vote.addVote
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addVote
     * @param options Request options.
     */
    async addIssueVote(options) {
        return this.Issue.Vote.addVote(options);
    }
    /**
     * A REST sub-resource representing the voters on the issue.
     * Alternative usage: Issue.Vote.getVotes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getVotes
     * @param options Request options.
     */
    async getIssueVotes(options) {
        return this.Issue.Vote.getVotes(options);
    }
    /**
     * Returns a list of all issue types visible to the user.
     * Alternative usage: Issue.Type.getTypes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getIssueAllTypes
     * @param options Request options.
     */
    async getIssueTypes(options) {
        return this.Issue.Type.getTypes(options);
    }
    /**
     * Creates an issue type from a JSON representation and adds the issue newly created issue type to the default issue type scheme.
     * Alternative usage: Issue.Type.createType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-createIssueType
     * @param options Request options.
     */
    async createIssueType(options) {
        return this.Issue.Type.createType(options);
    }
    /**
     * Returns a full representation of the issue type that has the given id.
     * Alternative usage: Issue.Type.getType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getIssueType
     * @param options Request options.
     */
    async getIssueType(options) {
        return this.Issue.Type.getType(options);
    }
    /**
    * Deletes the specified issue type. If the issue type has any associated issues, these issues will be migrated to the alternative issue type specified in the parameter.
    * You can determine the alternative issue types by calling the /rest/api/2/issuetype/{id}/alternatives resource.
    * Alternative usage: Issue.Type.deleteType
    * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-deleteIssueType
    * @param options Request options.
    */
    async deleteIssueType(options) {
        return this.Issue.Type.deleteType(options);
    }
    /**
     * Updates the specified issue type from a JSON representation.
     * Alternative usage: Issue.Type.updateType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-updateIssueType
     * @param options Request options.
     */
    async updateIssueType(options) {
        return this.Issue.Type.updateType(options);
    }
    /**
     * Returns a list of all alternative issue types for the given issue type id. The list will contain these issues types, to which issues assigned to the given issue type can be migrated.
     * The suitable alternatives are issue types which are assigned to the same workflow, the same field configuration and the same screen scheme.
     * Alternative usage: Issue.Type.getAlternativeTypes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getAlternativeIssueTypes
     * @param options Request options.
     */
    async getAlternativeIssueTypes(options) {
        return this.Issue.Type.getAlternativeTypes(options);
    }
    /**
     * Creates an issue type scheme from a JSON representation. (Admin required)
     * Alternative usage: Issue.Type.Scheme.createScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-createIssueTypeScheme
     * @param options Request options.
     */
    async createIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.createScheme(options);
    }
    /**
     * Returns a list of all issue type schemes visible to the user (must be admin).
     * All issue types associated with the scheme will only be returned if an additional query parameter is provided: expand=schemes.issueTypes.
     * Similarly, the default issue type associated with the scheme (if one exists) will only be returned if additional an query parameter is provided: expand=schemes.defaultIssueType.
     * Note that both query parameters can be used together: expand=schemes.issueTypes,schemes.defaultIssueType.
     * Alternative usage: Issue.Type.Scheme.getSchemes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getAllIssueTypeSchemes
     * @param options Request options.
     */
    async getIssueTypeSchemes(options) {
        return this.Issue.Type.Scheme.getSchemes(options);
    }
    /**
     * Returns a full representation of the issue type scheme that has the given id (must be admin).
     * Alternative usage: Issue.Type.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getIssueTypeScheme
     * @param options Request options.
     */
    async getIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.getScheme(options);
    }
    /**
     * Updates the specified issue type scheme from a JSON representation. (Admin required)
     * Alternative usage: Issue.Type.Scheme.updateScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-updateIssueTypeScheme
     * @param options Request options.
     */
    async updateIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.updateScheme(options);
    }
    /**
     * Deletes the specified issue type scheme.
     * Any projects associated with this IssueTypeScheme will be automatically associated with the global default IssueTypeScheme. (Admin required)
     * Alternative usage: Issue.Type.Scheme.deleteScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-deleteIssueTypeScheme
     * @param options Request options.
     */
    async deleteIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.deleteScheme(options);
    }
    /**
     * Adds additional projects to those already associated with the specified issue type scheme. (Admin required)
     * Alternative usage: Issue.Type.Scheme.Project.addAssociations
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-addProjectAssociationsToScheme
     * @param options Request options.
     */
    async addProjectAssociationsToIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.Project.addAssociations(options);
    }
    /**
     * For the specified issue type scheme, returns all of the associated projects. (Admin required)
     * Alternative usage: Issue.Type.Scheme.Project.getAssociations
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getAssociatedProjects
     * @param options Request options.
     */
    async getProjectAssociationsForIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.Project.getAssociations(options);
    }
    /**
     * Associates the given projects with the specified issue type scheme.
     * Any existing project-associations the issue type scheme has will be overwritten. (Admin required)
     * Alternative usage: Issue.Type.Scheme.Project.setAssociations
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-setProjectAssociationsForScheme
     * @param options Request options.
     */
    async setProjectAssociationsForIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.Project.setAssociations(options);
    }
    /**
     * Removes all project associations for the specified issue type scheme.
     * These projects revert to an association with the default/global issue type scheme.(Admin required)
     * Alternative usage: Issue.Type.Scheme.Project.removeAssociations
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-removeAllProjectAssociations
     * @param options Request options.
     */
    async removeProjectAssociationsForIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.Project.removeAssociations(options);
    }
    /**
     * For the specified issue type scheme, removes the given project association.
     * This project reverts to an association with the default/global issue type scheme.(Admin required)
     * Alternative usage: Issue.Type.Scheme.Project.removeAssociation
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-removeProjectAssociation
     * @param options Request options.
     */
    async removeProjectAssociationForIssueTypeScheme(options) {
        return this.Issue.Type.Scheme.Project.removeAssociation(options);
    }
    /**
     * Get a list of the transitions possible for this issue by the current user, along with fields that are required and their types.
     * Fields will only be returned if expand=transitions.fields.
     * The fields in the metadata correspond to the fields in the transition screen for that transition. Fields not in the screen will not be in the metadata.
     * Alternative usage: Issue.Transition.getTransitions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getTransitions
     * @param options Request options.
     */
    async getIssueTransitions(options) {
        return this.Issue.Transition.getTransitions(options);
    }
    /**
     * Perform a transition on an issue. When performing the transition you can update or set other issue fields.
     * The fields that can be set on transtion, in either the fields parameter or the update parameter can be determined using the /rest/api/2/issue/{issueIdOrKey}/transitions?expand=transitions.fields resource. If a field is not configured to appear on the transition screen, then it will not be in the transition metadata, and a field validation error will occur if it is submitted. The updateHistory param adds the issues retrieved by this method to the current user's issue history, if set to true (by default, the issue history does not include issues retrieved via the REST API). You can view the issue history in the JIRA application, via the Issues dropdown or by using the lastViewed JQL field in an issue search.
     * Alternative usage: Issue.Transition.performTransition
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-doTransition
     * @param options Request options.
     */
    async performIssueTransition(options) {
        return this.Issue.Transition.performTransition(options);
    }
    /**
     * Returns an issue's subtask list.
     * Alternative usage: Issue.SubTask.getSubTasks
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-getSubTasks
     * @param options Request options.
     */
    async getIssueSubTasks(options) {
        return this.Issue.SubTask.getSubTasks(options);
    }
    /**
     * Alternative usage: Issue.SubTask.canMoveSubTask
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-canMoveSubTask
     * @param options Request options.
     */
    async canMoveIssueSubTask(options) {
        return this.Issue.SubTask.canMoveSubTask(options);
    }
    /**
     * Alternative usage: Issue.SubTask.canMoveSubTask
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-canMoveSubTask
     * @param options Request options.
     */
    async moveIssueSubTasks(options) {
        return this.Issue.SubTask.moveSubTasks(options);
    }
    /**
     * Performs a search using JQL.
     * Alternative usage: Issue.Search.searchByJql
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#search-searchUsingSearchRequest
     * @param options Request options.
     */
    async searchIssuesByJql(options) {
        return this.Issue.Search.searchByJql(options);
    }
    /**
     * Returns a list of all resolutions.
     * Alternative usage: Issue.Resolution.getResolutions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#resolution-getResolutions
     * @param options Request options.
     */
    async getIssueResolutions(options) {
        return this.Issue.Resolution.getResolutions(options);
    }
    /**
     * Returns a resolution.
     * Alternative usage: Issue.Resolution.getResolution
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#resolution-getResolution
     * @param options Request options.
     */
    async getIssueResolution(options) {
        return this.Issue.Resolution.getResolution(options);
    }
    /**
     * Returns the keys of all properties for the issue identified by the key or by the id.
     * Alternative usage: Issue.Property.getPropertyKeys
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-getPropertiesKeys
     * @param options Request options.
     */
    async getIssuePropertyKeys(options) {
        return this.Issue.Property.getPropertyKeys(options);
    }
    /**
     * Removes the property from the issue identified by the key or by the id.
     * Ths user removing the property is required to have permissions to edit the issue.
     * Alternative usage: Issue.Property.deleteProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-deleteProperty
     * @param options Request options.
     */
    async deleteIssueProperty(options) {
        return this.Issue.Property.deleteProperty(options);
    }
    /**
     * Sets the value of the specified issue's property.
     * You can use this resource to store a custom data against the issue identified by the key or by the id.
     * The user who stores the data is required to have permissions to edit the issue.
     * Alternative usage: Issue.Property.setProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-setProperty
     * @param options Request options.
     */
    async setIssueProperty(options) {
        return this.Issue.Property.setProperty(options);
    }
    /**
     * Returns the value of the property with a given key from the issue identified by the key or by the id.
     * The user who retrieves the property is required to have permissions to read the issue.
     * Alternative usage: Issue.Property.getProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-getProperty
     * @param options Request options.
     */
    async getIssueProperty(options) {
        return this.Issue.Property.getProperty(options);
    }
    /**
     * Returns a list of all issue priorities.
     * Alternative usage: Issue.Priority.getPriorities
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#priority-getPriorities
     * @param options Request options.
     */
    async getIssuePriorities(options) {
        return this.Issue.Priority.getPriorities(options);
    }
    /**
     * Returns an issue priority.
     * Alternative usage: Issue.Priority.getPriority
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#priority-getPriority
     * @param options Request options.
     */
    async getIssuePriority(options) {
        return this.Issue.Priority.getPriority(options);
    }
    /**
     * Creates new priority scheme.
     * Alternative usage: Issue.Priority.Scheme.createScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-createPriorityScheme
     * @param options Request options.
     */
    async createIssuePriorityScheme(options) {
        return this.Issue.Priority.Scheme.createScheme(options);
    }
    /**
     * Returns all priority schemes.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=schemes.projectKeys.
     * Alternative usage: Issue.Priority.Scheme.getSchemes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-getPrioritySchemes
     * @param options Request options.
     */
    async getIssuePrioritySchemes(options) {
        return this.Issue.Priority.Scheme.getSchemes(options);
    }
    /**
     * Deletes a priority scheme.
     * All projects using deleted scheme will use default priority scheme afterwards. Default priority scheme can't be deleted.
     * Alternative usage: Issue.Priority.Scheme.deleteScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-deletePriorityScheme
     * @param options Request options.
     */
    async deleteIssuePriorityScheme(options) {
        return this.Issue.Priority.Scheme.deleteScheme(options);
    }
    /**
     * Updates a priority scheme.
     * Update will be rejected if issue migration would be needed as a result of scheme update.
     * Priority scheme update with migration is possible from the UI.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=projectKeys.
     * Alternative usage: Issue.Priority.Scheme.updateScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-updatePriorityScheme
     * @param options Request options.
     */
    async updateIssuePriorityScheme(options) {
        return this.Issue.Priority.Scheme.updateScheme(options);
    }
    /**
     * Gets a full representation of a priority scheme in JSON format.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=projectKeys.
     * Alternative usage: Issue.Priority.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-getPriorityScheme
     * @param options Request options.
     */
    async getIssuePriorityScheme(options) {
        return this.Issue.Priority.Scheme.getScheme(options);
    }
    /**
     * Returns the meta data for editing an issue. The fields in the editmeta correspond to the fields in the edit screen for the issue. Fields not in the screen will not be in the editmeta.
     * Alternative usage: Issue.Metadata.getEditMetadata
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getEditIssueMeta
     * @param options Request options.
     */
    async getEditIssueMetadata(options) {
        return this.Issue.Metadata.getEditMetadata(options);
    }
    /**
     * Returns the meta data for creating issues. This includes the available projects, issue types and fields, including field types and whether or not those fields are required. Projects will not be returned if the user does not have permission to create issues in that project.
     * The fields in the createmeta correspond to the fields in the create screen for the project/issuetype. Fields not in the screen will not be in the createmeta.
     * Fields will only be returned if expand=projects.issuetypes.fields.
     * The results can be filtered by project and/or issue type, given by the query params.
     * Alternative usage: Issue.Metadata.getCreateMetadata
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getCreateIssueMeta
     * @param options Request options.
     * @deprecated
     */
    async getCreateIssueMetadata(options) {
        return this.Issue.Metadata.getCreateMetadata(options);
    }
    /**
     * Creates an issue link between two issues. The user requires the link issue permission for the issue which will be linked to another issue. The specified link type in the request is used to create the link and will create a link from the first issue to the second issue using the outward description. It also create a link from the second issue to the first issue using the inward description of the issue link type. It will add the supplied comment to the first issue. The comment can have a restriction who can view it. If group is specified, only users of this group can view this comment, if roleLevel is specified only users who have the specified role can view this comment. The user who creates the issue link needs to belong to the specified group or have the specified role.
     * Alternative usage: Issue.Link.createLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-linkIssues
     * @param options Request options.
     */
    async createIssueLink(options) {
        return this.Issue.Link.createLink(options);
    }
    /**
     * Returns an issue link with the specified id.
     * Alternative usage: Issue.Link.getLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-getIssueLink
     * @param options Request options.
     */
    async getIssueLink(options) {
        return this.Issue.Link.getLink(options);
    }
    /**
     * Deletes an issue link with the specified id. To be able to delete an issue link you must be able to view both issues and must have the link issue permission for at least one of the issues.
     * Alternative usage: Issue.Link.deleteLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-deleteIssueLink
     * @param options Request options.
     */
    async deleteIssueLink(options) {
        return this.Issue.Link.deleteLink(options);
    }
    /**
     * A REST sub-resource representing the remote issue links on the issue.
     * Alternative usage: Issue.Link.Remote.getLinks
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getRemoteIssueLinks
     * @param options Request options.
     */
    async getRemoteIssueLinks(options) {
        return this.Issue.Link.Remote.getLinks(options);
    }
    /**
     * Creates or updates a remote issue link from a JSON representation. If a globalId is provided and a remote issue link exists with that globalId, the remote issue link is updated. Otherwise, the remote issue link is created.
     * Alternative usage: Issue.Link.Remote.createOrUpdateLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-createOrUpdateRemoteIssueLink
     * @param options Request options.
     */
    async createOrUpdateRemoteIssueLink(options) {
        return this.Issue.Link.Remote.createOrUpdateLink(options);
    }
    /**
     * Delete the remote issue link with the given global id on the issue.
     * Alternative usage: Issue.Link.Remote.deleteLinkByGlobalId
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteRemoteIssueLinkByGlobalId
     * @param options Request options.
     */
    async deleteRemoteIssueLinkByGlobalId(options) {
        return this.Issue.Link.Remote.deleteLinkByGlobalId(options);
    }
    /**
     * Get the remote issue link with the given id on the issue.
     * Alternative usage: Issue.Link.Remote.getLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getRemoteIssueLinkById
     * @param options Request options.
     */
    async getRemoteIssueLink(options) {
        return this.Issue.Link.Remote.getLink(options);
    }
    /**
     * Updates a remote issue link from a JSON representation. Any fields not provided are set to null.
     * Alternative usage: Issue.Link.Remote.updateLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateRemoteIssueLink
     * @param options Request options.
     */
    async updateRemoteIssueLink(options) {
        return this.Issue.Link.Remote.updateLink(options);
    }
    /**
     * Delete the remote issue link with the given id on the issue.
     * Alternative usage: Issue.Link.Remote.deleteLink
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteRemoteIssueLinkById
     * @param options Request options.
     */
    async deleteRemoteIssueLink(options) {
        return this.Issue.Link.Remote.deleteLink(options);
    }
    /**
     * Returns a list of available issue link types, if issue linking is enabled. Each issue link type has an id, a name and a label for the outward and inward link relationship.
     * Alternative usage: Issue.Link.Type.getTypes
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-getIssueLinkTypes
     * @param options Request options.
     */
    async getIssueLinkTypes(options) {
        return this.Issue.Link.Type.getTypes(options);
    }
    /**
     * Returns for a given issue link type id all information about this issue link type.
     * Alternative usage: Issue.Link.Type.getType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-getIssueLinkType
     * @param options Request options.
     */
    async getIssueLinkType(options) {
        return this.Issue.Link.Type.getType(options);
    }
    /**
     * Create a new issue link type.
     * Alternative usage: Issue.Link.Type.createType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-createIssueLinkType
     * @param options Request options.
     */
    async createIssueLinkType(options) {
        return this.Issue.Link.Type.createType(options);
    }
    /**
     * Delete the specified issue link type.
     * Alternative usage: Issue.Link.Type.deleteType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-deleteIssueLinkType
     * @param options Request options.
     */
    async deleteIssueLinkType(options) {
        return this.Issue.Link.Type.deleteType(options);
    }
    /**
     * Update the specified issue link type.
     * Alternative usage: Issue.Link.Type.updateType
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-updateIssueLinkType
     * @param options Request options.
     */
    async updateIssueLinkType(options) {
        return this.Issue.Link.Type.updateType(options);
    }
    /**
     * Returns a list of all fields, both System and Custom.
     * Alternative usage: Issue.Field.getFields
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#field-getFields
     * @param options Request options.
     */
    async getIssueFields(options) {
        return this.Issue.Field.getFields(options);
    }
    /**
     * Alternative: Issue.Field.Custom.getFields
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#customFields-getCustomFields
     * @param options Request options.
     */
    async getIssueCustomFields(options) {
        return this.Issue.Field.Custom.getFields(options);
    }
    /**
     * Returns a full representation of the Custom Field Option that has the given id.
     * Alternative: Issue.Field.Custom.Option.getOption
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#customFieldOption-getCustomFieldOption
     * @param options Request options.
     */
    async getIssueCustomFieldOption(options) {
        return this.Issue.Field.Custom.Option.getOption(options);
    }
    /**
     * Returns the meta-data for an attachment, including the URI of the actual attached file.
     * Alternative usage: Issue.Attachment.getAttachment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-getAttachment
     * @param options Request options.
     */
    async getIssueAttachment(options) {
        return this.Issue.Attachment.getAttachment(options);
    }
    /**
     * Remove an attachment from an issue.
     * Alternative usage: Issue.Attachment.removeAttachment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-removeAttachment
     * @param options Request options.
     */
    async removeIssueAttachment(options) {
        return this.Issue.Attachment.removeAttachment(options);
    }
    /**
     * Add one or more attachments to an issue.
     * Alternative usage: Issue.Attachment.addAttachment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/attachments-addAttachment
     * @param options Request options.
     */
    async addIssueAttachment(options) {
        return this.Issue.Attachment.addAttachment(options);
    }
    /**
     * Returns the meta information for an attachments, specifically if they are enabled and the maximum upload size allowed.
     * Alternative usage: Issue.Attachment.Metadata.getMetadata
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-getAttachmentMeta
     * @param options Request options.
     */
    async getIssueAttachmentMetadata(options) {
        return this.Issue.Attachment.Metadata.getMetadata(options);
    }
    /**
     * Returns the keys of all properties for the comment identified by the key or by the id.
     * Alternative usage: Issue.Comment.Property.getPropertyKeys
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-getPropertiesKeys
     * @param options Request options.
     */
    async getIssueCommentPropertyKeys(options) {
        return this.Issue.Comment.Property.getPropertyKeys(options);
    }
    /**
     * Removes the property from the comment identified by the key or by the id.
     * Ths user removing the property is required to have permissions to administer the comment.
     * Alternative usage: Issue.Comment.Property.deleteProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-deleteProperty
     * @param options Request options.
     */
    async deleteIssueCommentProperty(options) {
        return this.Issue.Comment.Property.deleteProperty(options);
    }
    /**
     * Sets the value of the specified comment's property.
     * You can use this resource to store a custom data against the comment identified by the key or by the id.
     * The user who stores the data is required to have permissions to administer the comment.
     * Alternative usage: Issue.Comment.Property.setProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-setProperty
     * @param options Request options.
     */
    async setIssueCommentProperty(options) {
        return this.Issue.Comment.Property.setProperty(options);
    }
    /**
     * Sets the value of the specified comment's property.
     * You can use this resource to store a custom data against the comment identified by the key or by the id.
     * The user who stores the data is required to have permissions to administer the comment.
     * Alternative usage: Issue.Comment.Property.setProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-setProperty
     * @param options Request options.
     */
    async getIssueCommentProperty(options) {
        return this.Issue.Comment.Property.getProperty(options);
    }
    /**
     * Returns an application property.
     * Alternative usage: Instance.getApplicationProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getProperty
     * @param options Request options.
     */
    async getApplicationProperty(options) {
        return this.Instance.getApplicationProperty(options);
    }
    /**
     * Modify an application property via PUT. The "value" field present in the PUT will override the existing value.
     * Alternative usage: Instance.setApplicationProperty
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-setPropertyViaRestfulTable
     * @param options Request options.
     */
    async setApplicationProperty(options) {
        return this.Instance.setApplicationProperty(options);
    }
    /**
     * Returns the properties that are displayed on the "General Configuration > Advanced Settings" page.
     * Alternative usage: Instance.getAdvancedSettings
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getAdvancedSettings
     * @param options Request options.
     */
    async getAdvancedSettings(options) {
        return this.Instance.getAdvancedSettings(options);
    }
    /**
     * Returns the properties that are displayed on the "General Configuration > Advanced Settings" page.
     * Alternative usage: Instance.getAdvancedSettings
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getAdvancedSettings
     * @param options Request options.
     */
    async getInstanceConfiguration(options) {
        return this.Instance.getConfiguration(options);
    }
    /**
     * Returns the properties that are displayed on the "General Configuration > Advanced Settings" page.
     * Alternative usage: Instance.getAdvancedSettings
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getAdvancedSettings
     * @param options Request options.
     */
    async getInstanceInfo(options) {
        return this.Instance.getInfo(options);
    }
    /**
     * Creates a group by given group parameter.
     * Alternative usage: Group.createGroup
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-createGroup
     * @param options Request options.
     */
    async createGroup(options) {
        return this.Group.createGroup(options);
    }
    /**
     * Deletes a group by given group parameter.
     * Alternative usage: Group.removeGroup
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-removeGroup
     * @param options Request options.
     */
    async removeGroup(options) {
        return this.Group.removeGroup(options);
    }
    /**
     * This resource returns a paginated list of users who are members of the specified group and its subgroups. Users in the page are ordered by user names. User of this resource is required to have sysadmin or admin permissions.
     * Alternative usage: Group.User.getUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-getUsersFromGroup
     * @param options Request options.
     */
    async getUsersFromGroup(options) {
        return this.Group.User.getUsers(options);
    }
    /**
     * Adds given user to a group.
     * Alternative usage: Group.User.addUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-addUserToGroup
     * @param options Request options.
     */
    async addUsersToGroup(options) {
        return this.Group.User.addUsers(options);
    }
    /**
     * Removes given user from a group.
     * Alternative usage: Group.User.removeUsers
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-removeUserFromGroup
     * @param options Request options.
     */
    async removeUsersFromGroup(options) {
        return this.Group.User.removeUsers(options);
    }
    /**
     * Creates a new filter, and returns newly created filter. Currently sets permissions just using the users default sharing permissions.
     * Alternative usage: Filter.createFilter
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-createFilter
     * @param options Request options.
     */
    async createFilter(options) {
        return this.Filter.createFilter(options);
    }
    /**
     * Returns a filter given an id.
     * Alternative usage: Filter.getFilter
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getFilter
     * @param options Request options.
     */
    async getFilter(options) {
        return this.Filter.getFilter(options);
    }
    /**
     * Updates an existing filter, and returns its new value.
     * Alternative usage: Filter.editFilter
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-editFilter
     * @param options Request options.
     */
    async editFilter(options) {
        return this.Filter.editFilter(options);
    }
    /**
     * Delete a filter.
     * Alternative usage: Filter.deleteFilter
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-deleteFilter
     * @param options Request options.
     */
    async deleteFilter(options) {
        return this.Filter.deleteFilter(options);
    }
    /**
     * Returns the favourite filters of the logged-in user.
     * Alternative usage: Filter.getMyFavouriteFilters
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getFavouriteFilters
     * @param options Request options.
     */
    async getMyFavouriteFilters(options) {
        return this.Filter.getMyFavouriteFilters(options);
    }
    /**
     * Returns the default share scope of the logged-in user.
     * Alternative usage: Filter.Share.getShareScope
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getDefaultShareScope
     * @param options Request options.
     */
    async getDefaultShareScope(options) {
        return this.Filter.Share.getShareScope(options);
    }
    /**
     * Sets the default share scope of the logged-in user. Available values are GLOBAL and PRIVATE.
     * Alternative usage: Filter.Share.setShareScope
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-setDefaultShareScope
     * @param options Request options.
     */
    async setDefaultShareScope(options) {
        return this.Filter.Share.setShareScope(options);
    }
    /**
     * Returns the default columns for the given filter. Currently logged in user will be used as the user making such request.
     * Alternative usage: Filter.Column.getDefaultColumns
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-defaultColumns
     * @param options Request options.
     */
    async getFilterDefaultColumns(options) {
        return this.Filter.Column.getDefaultColumns(options);
    }
    /**
     * Sets the default columns for the given filter.
     * Alternative usage: Filter.Column.setDefaultColumns
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-setColumns
     * @param options Request options.
     */
    async setFilterDefaultColumns(options) {
        return this.Filter.Column.setDefaultColumns(options);
    }
    /**
     * Resets the columns for the given filter such that the filter no longer has its own column config.
     * Alternative usage: Filter.Column.resetColumns
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-resetColumns
     * @param options Request options.
     */
    async resetFilterColumns(options) {
        return this.Filter.Column.resetColumns(options);
    }
    /**
     * Returns all share permissions of the given filter.
     * Alternative usage: Filter.Share.Permission.getSharePermissions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermissions
     * @param options Request options.
     */
    async getFilterSharePermissions(options) {
        return this.Filter.Share.Permission.getSharePermissions(options);
    }
    /**
     * Adds a share permissions to the given filter. Adding a global permission removes all previous permissions from the filter.
     * Alternative usage: Filter.Share.Permission.addSharePermission
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-addSharePermission
     * @param options Request options.
     */
    async addFilterSharePermission(options) {
        return this.Filter.Share.Permission.addSharePermission(options);
    }
    /**
     * Returns a single share permission of the given filter.
     * Alternative usage: Filter.Share.Permission.getSharePermissions
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermission
     * @param options Request options.
     */
    async getFilterSharePermission(options) {
        return this.Filter.Share.Permission.getSharePermission(options);
    }
    /**
     * Removes a share permissions from the given filter.
     * Alternative usage: Filter.Share.Permission.deleteSharePermission
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermission
     * @param options Request options.
     */
    async deleteFilterSharePermission(options) {
        return this.Filter.Share.Permission.deleteSharePermission(options);
    }
    /**
     * Returns a list of all dashboards, optionally filtering them.
     * Alternative usage: Dashboard.getDashboards
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#dashboard-list
     * @param options Request options.
     */
    async getAllDashboards(options) {
        return this.Dashboard.getDashboards(options);
    }
    /**
     * Returns a single dashboard.
     * Alternative usage: Dashboard.getDashboard
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#dashboard-getDashboard
     * @param options Request options.
     */
    async getDashboardById(options) {
        return this.Dashboard.getDashboard(options);
    }
    /**
     * Create a new workflow scheme.
     * The body contains a representation of the new scheme. Values not passed are assumed to be set to their defaults.
     * Alternative usage: Workflow.Scheme.createScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-createScheme
     * @param options Request options.
     */
    async createWorkflowScheme(options) {
        return this.Workflow.Scheme.createScheme(options);
    }
    /**
     * Returns the requested workflow scheme to the caller.
     * Alternative usage: Workflow.Scheme.getScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-getById
     * @param options Request options.
     */
    async getWorkflowScheme(options) {
        return this.Workflow.Scheme.getScheme(options);
    }
    /**
     * Delete the passed workflow scheme.
     * Alternative usage: Workflow.Scheme.deleteScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-deleteScheme
     * @param options Request options.
     */
    async deleteWorkflowScheme(options) {
        return this.Workflow.Scheme.deleteScheme(options);
    }
    /**
     * Update the passed workflow scheme.
     * The body of the request is a representation of the workflow scheme.
     * Values not passed are assumed to indicate no change for that field.
     * Alternative usage: Workflow.Scheme.updateScheme
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-update
     * @param options Request options.
     */
    async updateWorkflowScheme(options) {
        return this.Workflow.Scheme.updateScheme(options);
    }
    /**
     * Returns a list of all statuses.
     * Alternative usage: Workflow.Status.getStatuses
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#status-getStatuses
     * @param options Request options.
     */
    async getWorkflowStatuses(options) {
        return this.Workflow.Status.getStatuses(options);
    }
    /**
     * Returns a full representation of the Status having the given id or name.
     * Alternative usage: Workflow.Status.getStatus
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#status-getStatus
     * @param options Request options.
     */
    async getWorkflowStatus(options) {
        return this.Workflow.Status.getStatus(options);
    }
    /**
     * Returns a list of all status categories.
     * Alternative usage: Workflow.Status.Category.getCategories
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#statuscategory-getStatusCategories
     * @param options Request options.
     */
    async getWorkflowStatusCategories(options) {
        return this.Workflow.Status.Category.getCategories(options);
    }
    /**
     * Returns a full representation of the StatusCategory having the given id or key.
     * Alternative usage: Workflow.Status.Category.getCategory
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#statuscategory-getStatusCategory
     * @param options Request options.
     */
    async getWorkflowStatusCategory(options) {
        return this.Workflow.Status.Category.getCategory(options);
    }
    /**
     * Returns all workflows.
     * Alternative usage: Workflow.getWorkflows
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#workflow-getAllWorkflows
     * @param options Request options.
     */
    async getWorkflows(options) {
        return this.Workflow.getWorkflows(options);
    }
    /**
     * Returns all ApplicationRoles in the system.
     * Will also return an ETag header containing a version hash of the collection of ApplicationRoles.
     * Alternative usage: Application.Role.getRoles
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-getAll
     * @param options Request options.
     */
    async getApplicationRoles(options) {
        return this.Application.Role.getRoles(options);
    }
    /**
     * Returns the ApplicationRole with passed key if it exists.
     * Alternative usage: Application.Role.getRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-get
     * @param options Request options.
     */
    async getApplicationRole(options) {
        return this.Application.Role.getRole(options);
    }
    /**
     * Updates the ApplicationRole with the passed data.
     * Only the groups and default groups setting of the role may be updated.
     * Requests to change the key or the name of the role will be silently ignored.
     * Alternative usage: Application.Role.updateRole
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-put
     * @param options Request options.
     */
    async updateApplicationRole(options) {
        return this.Application.Role.updateRole(options);
    }
    /**
     * Creates an issue or a sub-task from a JSON representation.
     * The fields that can be set on create, in either the fields parameter or the update parameter can be determined using the /rest/api/2/issue/createmeta resource. If a field is not configured to appear on the create screen, then it will not be in the createmeta, and a field validation error will occur if it is submitted.
     * Creating a sub-task is similar to creating a regular issue, with two important differences:
     * the issueType field must correspond to a sub-task issue type (you can use /issue/createmeta to discover sub-task issue types), and you must provide a parent field in the issue create request containing the id or key of the parent issue.
     * Alternative usage: Issue.createIssue
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-createIssue
     * @param options Request options.
     */
    async createIssue(options) {
        return this.Issue.createIssue(options);
    }
    /**
     * Returns a full representation of the issue for the given issue key.
     * JIRA will attempt to identify the issue by the issueIdOrKey path parameter. This can be an issue id, or an issue key. If the issue cannot be found via an exact match, JIRA will also look for the issue in a case-insensitive way, or by looking to see if the issue was moved. In either of these cases, the request will proceed as normal (a 302 or other redirect will not be returned). The issue key contained in the response will indicate the current value of issue's key.
     * Alternative usage: Issue.getIssue
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssue
     * @param options Request options.
     */
    async getIssue(options) {
        return this.Issue.getIssue(options);
    }
    /**
     * Edits an issue from a JSON representation.
     * The issue can either be updated by setting explicit the field value(s) or by using an operation to change the field value.
     * The fields that can be updated, in either the fields parameter or the update parameter, can be determined using the /rest/api/2/issue/{issueIdOrKey}/editmeta resource.
     * If a field is not configured to appear on the edit screen, then it will not be in the editmeta, and a field validation error will occur if it is submitted.
     * Specifying a "field_id": field_value in the "fields" is a shorthand for a "set" operation in the "update" section.
     * Field should appear either in "fields" or "update", not in both.
     * Alternative usage: Issue.editIssue
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-editIssue
     * @param options Request options.
     */
    async editIssue(options) {
        return this.Issue.editIssue(options);
    }
    /**
     * Delete an issue. If the issue has subtasks you must set the parameter deleteSubtasks=true to delete the issue. You cannot delete an issue without its subtasks also being deleted.
     * Alternative usage: Issue.deleteIssue
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteIssue
     * @param options Request options.
     */
    async deleteIssue(options) {
        return this.Issue.deleteIssue(options);
    }
    /**
     * Assigns an issue to a user. You can use this resource to assign issues when the user submitting the request has the assign permission but not the edit issue permission. If the name is "-1" automatic assignee is used. A null name will remove the assignee.
     * Alternative usage: Issue.assignIssue
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-assign
     * @param options Request options.
     */
    async assignIssue(options) {
        return this.Issue.assignIssue(options);
    }
    /**
     * Sends a notification (email) to the list or recipients defined in the request.
     * Alternative usage: Issue.sendNotification
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-notify
     * @param options Request options.
     */
    async sendNotificationForIssue(options) {
        return this.Issue.sendNotification(options);
    }
    /**
     * Returns all comments for an issue.
     * Alternative usage: Issue.Comment.getComments
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getComments
     * @param options Request options.
     */
    async getIssueComments(options) {
        return this.Issue.Comment.getComments(options);
    }
    /**
     * Adds a new comment to an issue.
     * Alternative usage: Issue.Comment.addComment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addComment
     * @param options Request options.
     */
    async addIssueComment(options) {
        return this.Issue.Comment.addComment(options);
    }
    /**
     * Returns a single comment.
     * Alternative usage: Issue.Comment.getComment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getComment
     * @param options Request options.
     */
    async getIssueComment(options) {
        return this.Issue.Comment.getComment(options);
    }
    /**
     * Updates an existing comment using its JSON representation.
     * Alternative usage: Issue.Comment.updateComment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateComment
     * @param options Request options.
     */
    async updateIssueComment(options) {
        return this.Issue.Comment.updateComment(options);
    }
    /**
     * Deletes an existing comment.
     * Alternative usage: Issue.Comment.deleteComment
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteComment
     * @param options Request options.
     */
    async deleteIssueComment(options) {
        return this.Issue.Comment.deleteComment(options);
    }
}
class IssueGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Comment = new CommentGroup(implementation, this.getGlobalErrorStrategy);
        this.Metadata = new MetadataGroup(implementation, this.getGlobalErrorStrategy);
        this.Link = new LinkGroup(implementation, this.getGlobalErrorStrategy);
        this.Vote = new VoteGroup(implementation, this.getGlobalErrorStrategy);
        this.Watcher = new WatcherGroup(implementation, this.getGlobalErrorStrategy);
        this.Worklog = new WorklogGroup(implementation, this.getGlobalErrorStrategy);
        this.Transition = new TransitionGroup(implementation, this.getGlobalErrorStrategy);
        this.SubTask = new SubTaskGroup(implementation, this.getGlobalErrorStrategy);
        this.Field = new FieldGroup(implementation, this.getGlobalErrorStrategy);
        this.Attachment = new AttachmentGroup(implementation, this.getGlobalErrorStrategy);
        this.Type = new TypeGroup(implementation, this.getGlobalErrorStrategy);
        this.Priority = new PriorityGroup(implementation, this.getGlobalErrorStrategy);
        this.Resolution = new ResolutionGroup(implementation, this.getGlobalErrorStrategy);
        this.Search = new SearchGroup(implementation, this.getGlobalErrorStrategy);
        this.Property = new IssuePropertyGroup(implementation, this.getGlobalErrorStrategy);
        this.Security = new IssueSecurityGroup(implementation, this.getGlobalErrorStrategy);
        this.Notification = new IssueNotificationGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Creates an issue or a sub-task from a JSON representation.
     * The fields that can be set on create, in either the fields parameter or the update parameter can be determined using the /rest/api/2/issue/createmeta resource. If a field is not configured to appear on the create screen, then it will not be in the createmeta, and a field validation error will occur if it is submitted.
     * Creating a sub-task is similar to creating a regular issue, with two important differences:
     * the issueType field must correspond to a sub-task issue type (you can use /issue/createmeta to discover sub-task issue types), and you must provide a parent field in the issue create request containing the id or key of the parent issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-createIssue
     * @param options Request options.
     */
    async createIssue(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/issue/', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a full representation of the issue for the given issue key.
     * JIRA will attempt to identify the issue by the issueIdOrKey path parameter. This can be an issue id, or an issue key. If the issue cannot be found via an exact match, JIRA will also look for the issue in a case-insensitive way, or by looking to see if the issue was moved. In either of these cases, the request will proceed as normal (a 302 or other redirect will not be returned). The issue key contained in the response will indicate the current value of issue's key.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssue
     * @param options Request options.
     */
    async getIssue(options) {
        const { issueIdOrKey, expand, fields, properties, updateHistory, headers, errorStrategy } = options;
        const queryParams = { expand, fields, properties, updateHistory };
        const request = new Request('GET', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete an issue. If the issue has subtasks you must set the parameter deleteSubtasks=true to delete the issue. You cannot delete an issue without its subtasks also being deleted.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteIssue
     * @param options Request options.
     */
    async deleteIssue(options) {
        const { issueIdOrKey, headers, deleteSubtasks, errorStrategy } = options;
        const queryParams = { deleteSubtasks };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Edits an issue from a JSON representation.
     * The issue can either be updated by setting explicit the field value(s) or by using an operation to change the field value.
     * The fields that can be updated, in either the fields parameter or the update parameter, can be determined using the /rest/api/2/issue/{issueIdOrKey}/editmeta resource.
     * If a field is not configured to appear on the edit screen, then it will not be in the editmeta, and a field validation error will occur if it is submitted.
     * Specifying a "field_id": field_value in the "fields" is a shorthand for a "set" operation in the "update" section.
     * Field should appear either in "fields" or "update", not in both.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-editIssue
     * @param options Request options.
     */
    async editIssue(options) {
        const { issueIdOrKey, body, headers, notifyUsers, errorStrategy } = options;
        const queryParams = { notifyUsers };
        const request = new Request('PUT', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Assigns an issue to a user. You can use this resource to assign issues when the user submitting the request has the assign permission but not the edit issue permission. If the name is "-1" automatic assignee is used. A null name will remove the assignee.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-assign
     * @param options Request options.
     */
    async assignIssue(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/assignee`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Sends a notification (email) to the list or recipients defined in the request.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-notify
     * @param options Request options.
     */
    async sendNotification(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/notify`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class CommentGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Property = new IssueCommentPropertyGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns all comments for an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getComments
     * @param options Request options.
     */
    async getComments(options) {
        const { issueIdOrKey, headers, expand, startAt, maxResults, orderBy, errorStrategy } = options;
        const queryParams = { expand, startAt, maxResults, orderBy };
        const request = new Request('GET', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/comment`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds a new comment to an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addComment
     * @param options Request options.
     */
    async addComment(options) {
        const { body, issueIdOrKey, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('POST', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/comment`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a single comment.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getComment
     * @param options Request options.
     */
    async getComment(options) {
        const { issueIdOrKey, id, headers, expand, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/comment/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates an existing comment using its JSON representation.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateComment
     * @param options Request options.
     */
    async updateComment(options) {
        const { issueIdOrKey, id, body, headers, expand, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('PUT', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/comment/${encodeURIComponent(id)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes an existing comment.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteComment
     * @param options Request options.
     */
    async deleteComment(options) {
        const { issueIdOrKey, id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/comment/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class IssueCommentPropertyGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the keys of all properties for the comment identified by the key or by the id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-getPropertiesKeys
     * @param options Request options.
     */
    async getPropertyKeys(options) {
        const { commentId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/comment/${encodeURIComponent(commentId)}/properties`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes the property from the comment identified by the key or by the id.
     * Ths user removing the property is required to have permissions to administer the comment.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-deleteProperty
     * @param options Request options.
     */
    async deleteProperty(options) {
        const { commentId, propertyKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/comment/${encodeURIComponent(commentId)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Sets the value of the specified comment's property.
     * You can use this resource to store a custom data against the comment identified by the key or by the id.
     * The user who stores the data is required to have permissions to administer the comment.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-setProperty
     * @param options Request options.
     */
    async setProperty(options) {
        const { commentId, propertyKey, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/comment/${encodeURIComponent(commentId)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Sets the value of the specified comment's property.
     * You can use this resource to store a custom data against the comment identified by the key or by the id.
     * The user who stores the data is required to have permissions to administer the comment.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#comment/{commentId}/properties-setProperty
     * @param options Request options.
     */
    async getProperty(options) {
        const { commentId, propertyKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/comment/${encodeURIComponent(commentId)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class AttachmentGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Metadata = new AttachmentMetadataGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns the meta-data for an attachment, including the URI of the actual attached file.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-getAttachment
     * @param options Request options.
     */
    async getAttachment(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/attachment/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Remove an attachment from an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-removeAttachment
     * @param options Request options.
     */
    async removeAttachment(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/attachment/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Add one or more attachments to an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/attachments-addAttachment
     * @param options Request options.
     */
    async addAttachment(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const formBody = new FormData();
        formBody.setBufferImplementation(this.implementation);
        body.forEach(x => {
            formBody.append('file', new Blob(this.implementation, [x.content], {}), x.fileName);
        });
        const bodyBlob = formBody.getBlob();
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/attachments`, constructHeaders(bodyBlob, headers), bodyBlob);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class AttachmentMetadataGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the meta information for an attachments, specifically if they are enabled and the maximum upload size allowed.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#attachment-getAttachmentMeta
     * @param options Request options.
     */
    async getMetadata(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/attachment/meta', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class FieldGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Custom = new CustomGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a list of all fields, both System and Custom.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#field-getFields
     * @param options Request options.
     */
    async getFields(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/field', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class CustomGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Option = new OptionGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#customFields-getCustomFields
     * @param options Request options.
     */
    async getFields(options) {
        const { maxResults, startAt, search, projectIds, screenIds, types, headers, errorStrategy } = options ?? {};
        const queryParams = { maxResults, startAt, search, projectIds, screenIds, types };
        const request = new Request('GET', constructUrl('/rest/api/2/customFields', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class OptionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a full representation of the Custom Field Option that has the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#customFieldOption-getCustomFieldOption
     * @param options Request options.
     */
    async getOption(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/customFieldOption/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class LinkGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Type = new LinkTypeGroup(implementation, this.getGlobalErrorStrategy);
        this.Remote = new RemoteGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Creates an issue link between two issues. The user requires the link issue permission for the issue which will be linked to another issue. The specified link type in the request is used to create the link and will create a link from the first issue to the second issue using the outward description. It also create a link from the second issue to the first issue using the inward description of the issue link type. It will add the supplied comment to the first issue. The comment can have a restriction who can view it. If group is specified, only users of this group can view this comment, if roleLevel is specified only users who have the specified role can view this comment. The user who creates the issue link needs to belong to the specified group or have the specified role.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-linkIssues
     * @param options Request options.
     */
    async createLink(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/issueLink', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns an issue link with the specified id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-getIssueLink
     * @param options Request options.
     */
    async getLink(options) {
        const { linkId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issueLink/${encodeURIComponent(linkId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes an issue link with the specified id. To be able to delete an issue link you must be able to view both issues and must have the link issue permission for at least one of the issues.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLink-deleteIssueLink
     * @param options Request options.
     */
    async deleteLink(options) {
        const { linkId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issueLink/${encodeURIComponent(linkId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class LinkTypeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a list of available issue link types, if issue linking is enabled. Each issue link type has an id, a name and a label for the outward and inward link relationship.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-getIssueLinkTypes
     * @param options Request options.
     */
    async getTypes(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/issueLinkType', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns for a given issue link type id all information about this issue link type.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-getIssueLinkType
     * @param options Request options.
     */
    async getType(options) {
        const { issueLinkTypeId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issueLinkType/${encodeURIComponent(issueLinkTypeId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Create a new issue link type.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-createIssueLinkType
     * @param options Request options.
     */
    async createType(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/issueLinkType/', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete the specified issue link type.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-deleteIssueLinkType
     * @param options Request options.
     */
    async deleteType(options) {
        const { issueLinkTypeId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issueLinkType/${encodeURIComponent(issueLinkTypeId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Update the specified issue link type.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issueLinkType-updateIssueLinkType
     * @param options Request options.
     */
    async updateType(options) {
        const { issueLinkTypeId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issueLinkType/${encodeURIComponent(issueLinkTypeId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RemoteGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * A REST sub-resource representing the remote issue links on the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getRemoteIssueLinks
     * @param options Request options.
     */
    async getLinks(options) {
        const { issueIdOrKey, globalId, headers, errorStrategy } = options;
        const queryParams = { globalId };
        const request = new Request('GET', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates or updates a remote issue link from a JSON representation. If a globalId is provided and a remote issue link exists with that globalId, the remote issue link is updated. Otherwise, the remote issue link is created.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-createOrUpdateRemoteIssueLink
     * @param options Request options.
     */
    async createOrUpdateLink(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete the remote issue link with the given global id on the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteRemoteIssueLinkByGlobalId
     * @param options Request options.
     */
    async deleteLinkByGlobalId(options) {
        const { issueIdOrKey, globalId, headers, errorStrategy } = options;
        const queryParams = { globalId };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Get the remote issue link with the given id on the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getRemoteIssueLinkById
     * @param options Request options.
     */
    async getLink(options) {
        const { issueIdOrKey, linkId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink/${encodeURIComponent(linkId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates a remote issue link from a JSON representation. Any fields not provided are set to null.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateRemoteIssueLink
     * @param options Request options.
     */
    async updateLink(options) {
        const { issueIdOrKey, linkId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink/${encodeURIComponent(linkId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Delete the remote issue link with the given id on the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteRemoteIssueLinkById
     * @param options Request options.
     */
    async deleteLink(options) {
        const { issueIdOrKey, linkId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/remotelink/${encodeURIComponent(linkId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class MetadataGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the meta data for editing an issue. The fields in the editmeta correspond to the fields in the edit screen for the issue. Fields not in the screen will not be in the editmeta.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getEditIssueMeta
     * @param options Request options.
     */
    async getEditMetadata(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/editmeta`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the meta data for creating issues. This includes the available projects, issue types and fields, including field types and whether or not those fields are required. Projects will not be returned if the user does not have permission to create issues in that project.
     * The fields in the createmeta correspond to the fields in the create screen for the project/issuetype. Fields not in the screen will not be in the createmeta.
     * Fields will only be returned if expand=projects.issuetypes.fields.
     * The results can be filtered by project and/or issue type, given by the query params.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getCreateIssueMeta
     * @param options Request options.
     * @deprecated
     */
    async getCreateMetadata(options) {
        const { expand, issuetypeIds, issuetypeNames, projectIds, projectKeys, headers, errorStrategy } = options ?? {};
        const queryParams = { expand, issuetypeIds, issuetypeNames, projectIds, projectKeys };
        const request = new Request('GET', constructUrl('/rest/api/2/issue/createmeta', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class PriorityGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new IssuePrioritySchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a list of all issue priorities.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#priority-getPriorities
     * @param options Request options.
     */
    async getPriorities(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/priority', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns an issue priority.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#priority-getPriority
     * @param options Request options.
     */
    async getPriority(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/priority/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class IssuePrioritySchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Creates new priority scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-createPriorityScheme
     * @param options Request options.
     */
    async createScheme(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/priorityschemes', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns all priority schemes.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=schemes.projectKeys.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-getPrioritySchemes
     * @param options Request options.
     */
    async getSchemes(options) {
        const { startAt, maxResults, expand, headers, errorStrategy } = options ?? {};
        const queryParams = { startAt, maxResults, expand };
        const request = new Request('GET', constructUrl('/rest/api/2/priorityschemes', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a priority scheme.
     * All projects using deleted scheme will use default priority scheme afterwards. Default priority scheme can't be deleted.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-deletePriorityScheme
     * @param options Request options.
     */
    async deleteScheme(options) {
        const { schemeId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/priorityschemes/${encodeURIComponent(schemeId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Updates a priority scheme.
     * Update will be rejected if issue migration would be needed as a result of scheme update.
     * Priority scheme update with migration is possible from the UI.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=projectKeys.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-updatePriorityScheme
     * @param options Request options.
     */
    async updateScheme(options) {
        const { expand, schemeId, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('PUT', constructUrl(`/rest/api/2/priorityschemes/${encodeURIComponent(schemeId)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Gets a full representation of a priority scheme in JSON format.
     * All project keys associated with the priority scheme will only be returned if additional query parameter is provided expand=projectKeys.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#priorityschemes-getPriorityScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { expand, schemeId, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/priorityschemes/${encodeURIComponent(schemeId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class IssuePropertyGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the keys of all properties for the issue identified by the key or by the id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-getPropertiesKeys
     * @param options Request options.
     */
    async getPropertyKeys(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/properties`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes the property from the issue identified by the key or by the id.
     * Ths user removing the property is required to have permissions to edit the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-deleteProperty
     * @param options Request options.
     */
    async deleteProperty(options) {
        const { issueIdOrKey, propertyKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Sets the value of the specified issue's property.
     * You can use this resource to store a custom data against the issue identified by the key or by the id.
     * The user who stores the data is required to have permissions to edit the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-setProperty
     * @param options Request options.
     */
    async setProperty(options) {
        const { issueIdOrKey, propertyKey, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns the value of the property with a given key from the issue identified by the key or by the id.
     * The user who retrieves the property is required to have permissions to read the issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issue/{issueIdOrKey}/properties-getProperty
     * @param options Request options.
     */
    async getProperty(options) {
        const { issueIdOrKey, propertyKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/properties/${encodeURIComponent(propertyKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ResolutionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a list of all resolutions.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#resolution-getResolutions
     * @param options Request options.
     */
    async getResolutions(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/resolution', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a resolution.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#resolution-getResolution
     * @param options Request options.
     */
    async getResolution(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/resolution/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class SearchGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a resolution.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#resolution-getResolution
     * @param options Request options.
     */
    async searchByJql(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/search', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class SubTaskGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns an issue's subtask list.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-getSubTasks
     * @param options Request options.
     */
    async getSubTasks(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/subtask`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-canMoveSubTask
     * @param options Request options.
     */
    async canMoveSubTask(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/subtask/move`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Reorders an issue's subtasks by moving the subtask at index "from" to index "to".
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-moveSubTasks
     * @param options Request options.
     */
    async moveSubTasks(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/subtask/move`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class TransitionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Reorders an issue's subtasks by moving the subtask at index "from" to index "to".
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue/{issueIdOrKey}/subtask-moveSubTasks
     * @param options Request options.
     */
    async getTransitions(options) {
        const { issueIdOrKey, headers, expand, transitionId, errorStrategy } = options;
        const queryParams = { expand, transitionId };
        const request = new Request('GET', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/transitions`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Perform a transition on an issue. When performing the transition you can update or set other issue fields.
     * The fields that can be set on transtion, in either the fields parameter or the update parameter can be determined using the /rest/api/2/issue/{issueIdOrKey}/transitions?expand=transitions.fields resource. If a field is not configured to appear on the transition screen, then it will not be in the transition metadata, and a field validation error will occur if it is submitted. The updateHistory param adds the issues retrieved by this method to the current user's issue history, if set to true (by default, the issue history does not include issues retrieved via the REST API). You can view the issue history in the JIRA application, via the Issues dropdown or by using the lastViewed JQL field in an issue search.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-doTransition
     * @param options Request options.
     */
    async performTransition(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/transitions`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class TypeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new IssueTypeSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a list of all issue types visible to the user.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getIssueAllTypes
     * @param options Request options.
     */
    async getTypes(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/issuetype', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates an issue type from a JSON representation and adds the issue newly created issue type to the default issue type scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-createIssueType
     * @param options Request options.
     */
    async createType(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/issuetype', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a full representation of the issue type that has the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getIssueType
     * @param options Request options.
     */
    async getType(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issuetype/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes the specified issue type. If the issue type has any associated issues, these issues will be migrated to the alternative issue type specified in the parameter.
     * You can determine the alternative issue types by calling the /rest/api/2/issuetype/{id}/alternatives resource.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-deleteIssueType
     * @param options Request options.
     */
    async deleteType(options) {
        const { id, alternativeIssueTypeId, headers, errorStrategy } = options;
        const queryParams = { alternativeIssueTypeId };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/issuetype/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Updates the specified issue type from a JSON representation.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-updateIssueType
     * @param options Request options.
     */
    async updateType(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issuetype/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of all alternative issue types for the given issue type id. The list will contain these issues types, to which issues assigned to the given issue type can be migrated.
     * The suitable alternatives are issue types which are assigned to the same workflow, the same field configuration and the same screen scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issuetype-getAlternativeIssueTypes
     * @param options Request options.
     */
    async getAlternativeTypes(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issuetype/${encodeURIComponent(id)}/alternatives`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class IssueTypeSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Project = new IssueTypeSchemeProjectGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Creates an issue type scheme from a JSON representation. (Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-createIssueTypeScheme
     * @param options Request options.
     */
    async createScheme(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/issuetypescheme', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of all issue type schemes visible to the user (must be admin).
     * All issue types associated with the scheme will only be returned if an additional query parameter is provided: expand=schemes.issueTypes.
     * Similarly, the default issue type associated with the scheme (if one exists) will only be returned if additional an query parameter is provided: expand=schemes.defaultIssueType.
     * Note that both query parameters can be used together: expand=schemes.issueTypes,schemes.defaultIssueType.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getAllIssueTypeSchemes
     * @param options Request options.
     */
    async getSchemes(options) {
        const { expand, headers, errorStrategy } = options ?? {};
        const queryParams = { expand };
        const request = new Request('GET', constructUrl('/rest/api/2/issuetypescheme', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a full representation of the issue type scheme that has the given id (must be admin).
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getIssueTypeScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { schemeId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates the specified issue type scheme from a JSON representation. (Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-updateIssueTypeScheme
     * @param options Request options.
     */
    async updateScheme(options) {
        const { schemeId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes the specified issue type scheme.
     * Any projects associated with this IssueTypeScheme will be automatically associated with the global default IssueTypeScheme. (Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-deleteIssueTypeScheme
     * @param options Request options.
     */
    async deleteScheme(options) {
        const { schemeId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class IssueTypeSchemeProjectGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Perform a transition on an issue. When performing the transition you can update or set other issue fields.
     * The fields that can be set on transtion, in either the fields parameter or the update parameter can be determined using the /rest/api/2/issue/{issueIdOrKey}/transitions?expand=transitions.fields resource. If a field is not configured to appear on the transition screen, then it will not be in the transition metadata, and a field validation error will occur if it is submitted. The updateHistory param adds the issues retrieved by this method to the current user's issue history, if set to true (by default, the issue history does not include issues retrieved via the REST API). You can view the issue history in the JIRA application, via the Issues dropdown or by using the lastViewed JQL field in an issue search.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-doTransition
     * @param options Request options.
     */
    async addAssociations(options) {
        const { schemeId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}/associations`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * For the specified issue type scheme, returns all of the associated projects. (Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-getAssociatedProjects
     * @param options Request options.
     */
    async getAssociations(options) {
        const { schemeId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}/associations`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Associates the given projects with the specified issue type scheme.
     * Any existing project-associations the issue type scheme has will be overwritten. (Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-setProjectAssociationsForScheme
     * @param options Request options.
     */
    async setAssociations(options) {
        const { schemeId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}/associations`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Removes all project associations for the specified issue type scheme.
     * These projects revert to an association with the default/global issue type scheme.(Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-removeAllProjectAssociations
     * @param options Request options.
     */
    async removeAssociations(options) {
        const { schemeId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}/associations`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * For the specified issue type scheme, removes the given project association.
     * This project reverts to an association with the default/global issue type scheme.(Admin required)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuetypescheme-removeProjectAssociation
     * @param options Request options.
     */
    async removeAssociation(options) {
        const { schemeId, projIdOrKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issuetypescheme/${encodeURIComponent(schemeId)}/associations/${encodeURIComponent(projIdOrKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class VoteGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Remove your vote from an issue. (i.e. "unvote").
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-removeVote
     * @param options Request options.
     */
    async removeVote(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/votes`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Cast your vote in favour of an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addVote
     * @param options Request options.
     */
    async addVote(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/votes`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
    * A REST sub-resource representing the voters on the issue.
    * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getVotes
    * @param options Request options.
    */
    async getVotes(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/votes`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class WatcherGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the list of watchers for the issue with the given key.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssueWatchers
     * @param options Request options.
     */
    async getWatchers(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/watchers`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds a user to an issue's watcher list.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addWatcher
     * @param options Request options.
     */
    async addWatcher(options) {
        const { issueIdOrKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/watchers`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Removes a user from an issue's watcher list.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-removeWatcher
     * @param options Request options.
     */
    async removeWatcher(options) {
        const { issueIdOrKey, username, headers, errorStrategy } = options;
        const queryParams = { username };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/watchers`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class WorklogGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all work logs for an issue. Work logs won't be returned if the Log work field is hidden for the project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getIssueWorklog
     * @param options Request options.
     */
    async getWorklogs(options) {
        const { issueIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/worklog`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds a new worklog entry to an issue.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-addWorklog
     * @param options Request options.
     */
    async addWorklog(options) {
        const { issueIdOrKey, body, headers, adjustEstimate, newEstimate, reduceBy, errorStrategy } = options;
        const queryParams = { adjustEstimate, newEstimate, reduceBy };
        const request = new Request('POST', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/worklog`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a specific worklog. The work log won't be returned if the Log work field is hidden for the project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-getWorklog
     * @param options Request options.
     */
    async getWorklog(options) {
        const { issueIdOrKey, id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/worklog/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates an existing worklog entry. Fields possible for editing are: comment, visibility, started, timeSpent and timeSpentSeconds. Either timeSpent or timeSpentSeconds can be set. Fields which are not set will not be updated. For a request to be valid, it has to have at least one field change.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-updateWorklog
     * @param options Request options.
     */
    async updateWorklog(options) {
        const { issueIdOrKey, id, body, headers, newEstimate, adjustEstimate, errorStrategy } = options;
        const queryParams = { adjustEstimate, newEstimate };
        const request = new Request('PUT', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/worklog/${encodeURIComponent(id)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes an existing worklog entry.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#issue-deleteWorklog
     * @param options Request options.
     */
    async deleteWorklog(options) {
        const { issueIdOrKey, id, headers, adjustEstimate, newEstimate, increaseBy, errorStrategy } = options;
        const queryParams = { adjustEstimate, newEstimate, increaseBy };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/issue/${encodeURIComponent(issueIdOrKey)}/worklog/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns worklogs id and delete time of worklogs that was deleted since given time. The returns set of worklogs is limited to 1000 elements. This API will not return worklogs deleted during last minute.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getIdsOfWorklogsDeletedSince
     * @param options Request options.
     */
    async getDeletedWorklogIds(options) {
        const { since, headers, errorStrategy } = options ?? {};
        const queryParams = { since };
        const request = new Request('GET', constructUrl('/rest/api/2/worklog/deleted', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns worklogs for given worklog ids. Only worklogs to which the calling user has permissions, will be included in the result. The returns set of worklogs is limited to 1000 elements.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getWorklogsForIds
     * @param options Request options.
     */
    async getWorklogsForIds(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/worklog/list', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns worklogs id and update time of worklogs that was updated since given time. The returns set of worklogs is limited to 1000 elements. This API will not return worklogs updated during last minute.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#worklog-getIdsOfWorklogsModifiedSince
     * @param options Request options.
     */
    async getUpdatedWorklogIds(options) {
        const { since, headers, errorStrategy } = options ?? {};
        const queryParams = { since };
        const request = new Request('GET', constructUrl('/rest/api/2/worklog/updated', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class IssueSecurityGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new IssueSecuritySchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class IssueSecuritySchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all issue security schemes that are defined.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuesecurityschemes-getIssueSecuritySchemes
     * @param options Request options.
     */
    async getSchemes(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/issuesecurityschemes', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the issue security scheme along with that are defined.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#issuesecurityschemes-getIssueSecurityScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/issuesecurityschemes/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class IssueNotificationGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new IssueNotificationSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class IssueNotificationSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a paginated list of notification schemes.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#notificationscheme-getNotificationSchemes
     * @param options Request options.
     */
    async getSchemes(options) {
        const { startAt, maxResults, expand, headers, errorStrategy } = options ?? {};
        const queryParams = { startAt, maxResults, expand };
        const request = new Request('GET', constructUrl('/rest/api/2/notificationscheme', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a full representation of the notification scheme for the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#notificationscheme-getNotificationScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { id, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/notificationscheme/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ApplicationGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Role = new ApplicationRoleGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ApplicationRoleGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all ApplicationRoles in the system.
     * Will also return an ETag header containing a version hash of the collection of ApplicationRoles.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-getAll
     * @param options Request options.
     */
    async getRoles(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/applicationrole', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the ApplicationRole with passed key if it exists.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-get
     * @param options Request options.
     */
    async getRole(options) {
        const { key, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/applicationrole/${encodeURIComponent(key)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates the ApplicationRole with the passed data.
     * Only the groups and default groups setting of the role may be updated.
     * Requests to change the key or the name of the role will be silently ignored.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#applicationrole-put
     * @param options Request options.
     */
    async updateRole(options) {
        const { key, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/applicationrole/${encodeURIComponent(key)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class WorkflowGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Status = new WorkflowStatusGroup(implementation, this.getGlobalErrorStrategy);
        this.Scheme = new WorkflowSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns all workflows.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#workflow-getAllWorkflows
     * @param options Request options.
     */
    async getWorkflows(options) {
        const { workflowName, headers, errorStrategy } = options ?? {};
        const queryParams = { workflowName };
        const request = new Request('GET', constructUrl('/rest/api/2/workflow', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class WorkflowStatusGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Category = new StatusCategoryGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a list of all statuses.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#status-getStatuses
     * @param options Request options.
     */
    async getStatuses(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/status', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a full representation of the Status having the given id or name.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#status-getStatus
     * @param options Request options.
     */
    async getStatus(options) {
        const { idOrName, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/status/${encodeURIComponent(idOrName)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class StatusCategoryGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a list of all status categories.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#statuscategory-getStatusCategories
     * @param options Request options.
     */
    async getCategories(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/statuscategory', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of all status categories.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#statuscategory-getStatusCategories
     * @param options Request options.
     */
    async getCategory(options) {
        const { idOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/statuscategory/${encodeURIComponent(idOrKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class WorkflowSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Create a new workflow scheme.
     * The body contains a representation of the new scheme. Values not passed are assumed to be set to their defaults.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-createScheme
     * @param options Request options.
     */
    async createScheme(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/workflowscheme', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the requested workflow scheme to the caller.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-getById
     * @param options Request options.
     */
    async getScheme(options) {
        const { id, returnDraftIfExists, headers, errorStrategy } = options;
        const queryParams = { returnDraftIfExists };
        const request = new Request('GET', constructUrl(`/rest/api/2/workflowscheme/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the requested workflow scheme to the caller.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-getById
     * @param options Request options.
     */
    async deleteScheme(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/workflowscheme/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Update the passed workflow scheme.
     * The body of the request is a representation of the workflow scheme.
     * Values not passed are assumed to indicate no change for that field.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#workflowscheme-update
     * @param options Request options.
     */
    async updateScheme(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/workflowscheme/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class DashboardGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a list of all dashboards, optionally filtering them.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#dashboard-list
     * @param options Request options.
     */
    async getDashboards(options) {
        const { filter, startAt, maxResults, headers, errorStrategy } = options ?? {};
        const queryParams = { filter, startAt, maxResults };
        const request = new Request('GET', constructUrl('/rest/api/2/dashboard', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a single dashboard.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#dashboard-getDashboard
     * @param options Request options.
     */
    async getDashboard(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/dashboard/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class FilterGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Column = new FilterColumnGroup(implementation, this.getGlobalErrorStrategy);
        this.Share = new FilterShareGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a filter given an id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getFilter
     * @param options Request options.
     */
    async getFilter(options) {
        const { id, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/filter/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the favourite filters of the logged-in user.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getFavouriteFilters
     * @param options Request options.
     */
    async getMyFavouriteFilters(options) {
        const { expand, headers, errorStrategy } = options ?? {};
        const queryParams = { expand };
        const request = new Request('GET', constructUrl('/rest/api/2/filter/favourite', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates a new filter, and returns newly created filter. Currently sets permissions just using the users default sharing permissions.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-createFilter
     * @param options Request options.
     */
    async createFilter(options) {
        const { expand, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('POST', constructUrl('/rest/api/2/filter', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates an existing filter, and returns its new value.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-editFilter
     * @param options Request options.
     */
    async editFilter(options) {
        const { expand, id, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('PUT', constructUrl(`/rest/api/2/filter/${encodeURIComponent(id)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete a filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-deleteFilter
     * @param options Request options.
     */
    async deleteFilter(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/filter/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class FilterColumnGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the default columns for the given filter. Currently logged in user will be used as the user making such request.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-defaultColumns
     * @param options Request options.
     */
    async getDefaultColumns(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/filter/${encodeURIComponent(id)}/columns`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Sets the default columns for the given filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-setColumns
     * @param options Request options.
     */
    async setDefaultColumns(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/filter/${encodeURIComponent(id)}/columns`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Resets the columns for the given filter such that the filter no longer has its own column config.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-resetColumns
     * @param options Request options.
     */
    async resetColumns(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/filter/${encodeURIComponent(id)}/columns`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class FilterShareGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Permission = new SharePermissionGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns the default share scope of the logged-in user.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getDefaultShareScope
     * @param options Request options.
     */
    async getShareScope(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/filter/defaultShareScope', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Sets the default share scope of the logged-in user. Available values are GLOBAL and PRIVATE.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-setDefaultShareScope
     * @param options Request options.
     */
    async setShareScope(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('PUT', '/rest/api/2/filter/defaultShareScope', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class SharePermissionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all share permissions of the given filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermissions
     * @param options Request options.
     */
    async getSharePermissions(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/filter/${encodeURIComponent(id)}/permission`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds a share permissions to the given filter. Adding a global permission removes all previous permissions from the filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-addSharePermission
     * @param options Request options.
     */
    async addSharePermission(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/filter/${encodeURIComponent(id)}/permission`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a single share permission of the given filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermission
     * @param options Request options.
     */
    async getSharePermission(options) {
        const { id, permissionId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/filter/${encodeURIComponent(id)}/permission/${encodeURIComponent(permissionId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes a share permissions from the given filter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#filter-getSharePermission
     * @param options Request options.
     */
    async deleteSharePermission(options) {
        const { id, permissionId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/filter/${encodeURIComponent(id)}/permission/${encodeURIComponent(permissionId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class GroupGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.User = new GroupUserGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Creates a group by given group parameter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-createGroup
     * @param options Request options.
     */
    async createGroup(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/group', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a group by given group parameter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-removeGroup
     * @param options Request options.
     */
    async removeGroup(options) {
        const { groupname, swapGroup, headers, errorStrategy } = options ?? {};
        const queryParams = { groupname, swapGroup };
        const request = new Request('DELETE', constructUrl('/rest/api/2/group', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class GroupUserGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * This resource returns a paginated list of users who are members of the specified group and its subgroups. Users in the page are ordered by user names. User of this resource is required to have sysadmin or admin permissions.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-getUsersFromGroup
     * @param options Request options.
     */
    async getUsers(options) {
        const { groupname, includeInactiveUsers, maxResults, startAt, headers, errorStrategy } = options;
        const queryParams = { groupname, includeInactiveUsers, maxResults, startAt };
        const request = new Request('GET', constructUrl('/rest/api/2/group/member', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds given user to a group.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-addUserToGroup
     * @param options Request options.
     */
    async addUsers(options) {
        const { groupname, body, headers, errorStrategy } = options;
        const queryParams = { groupname };
        const request = new Request('POST', constructUrl('/rest/api/2/group/user', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes given user from a group.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#group-removeUserFromGroup
     * @param options Request options.
     */
    async removeUsers(options) {
        const { groupname, username, headers, errorStrategy } = options ?? {};
        const queryParams = { groupname, username };
        const request = new Request('DELETE', constructUrl('/rest/api/2/group/user', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class InstanceGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns an application property.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getProperty
     * @param options Request options.
     */
    async getApplicationProperty(options) {
        const { key, keyFilter, permissionLevel, headers, errorStrategy } = options ?? {};
        const queryParams = { key, keyFilter, permissionLevel };
        const request = new Request('GET', constructUrl('/rest/api/2/application-properties', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Modify an application property via PUT. The "value" field present in the PUT will override the existing value.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-setPropertyViaRestfulTable
     * @param options Request options.
     */
    async setApplicationProperty(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/application-properties/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the properties that are displayed on the "General Configuration > Advanced Settings" page.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#application-properties-getAdvancedSettings
     * @param options Request options.
     */
    async getAdvancedSettings(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/application-properties/advanced-settings', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the information if the optional features in JIRA are enabled or disabled. If the time tracking is enabled, it also returns the detailed information about time tracking configuration.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#configuration-getConfiguration
     * @param options Request options.
     */
    async getConfiguration(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/configuration', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns general information about the current JIRA server.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#serverInfo-getServerInfo
     * @param options Request options.
     */
    async getInfo(options) {
        const { headers, errorStrategy, doHealthCheck } = options ?? {};
        const queryParams = { doHealthCheck };
        const request = new Request('GET', constructUrl('/rest/api/2/serverInfo', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class MyselfGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Sets preference of the currently logged in user.
     * Preference key must be provided as input parameters (key).
     * Value must be provided as post body. If key or value parameter is not provided - status code 404.
     * If preference is set - status code 204.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-setPreference
     * @param options Request options.
     */
    async setPreference(options) {
        const { key, body, headers, errorStrategy } = options;
        const queryParams = { key };
        const request = new Request('PUT', constructUrl('/rest/api/2/mypreferences', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Removes preference of the currently logged in user. Preference key must be provided as input parameters (key).
     * If key parameter is not provided or wrong - status code 404.
     * If preference is unset - status code 204.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-removePreference
     * @param options Request options.
     */
    async removePreference(options) {
        const { key, headers, errorStrategy } = options;
        const queryParams = { key };
        const request = new Request('DELETE', constructUrl('/rest/api/2/mypreferences', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns preference of the currently logged in user.
     * Preference key must be provided as input parameter (key).
     * The value is returned exactly as it is. If key parameter is not provided or wrong - status code 404.
     * If value is found - status code 200.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#mypreferences-getPreference
     * @param options Request options.
     */
    async getPreference(options) {
        const { key, headers, errorStrategy } = options;
        const queryParams = { key };
        const request = new Request('GET', constructUrl('/rest/api/2/mypreferences', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns currently logged in user.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#myself-getUser
     * @param options Request options.
     */
    async getCurrentUser(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/myself', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class PermissionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new PermissionSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns all permissions in the system and whether the currently logged in user has them.
     * You can optionally provide a specific context to get permissions for (projectKey OR projectId OR issueKey OR issueId)
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#-getPermissions
     * @param options Request options.
     */
    async getMyPermissions(options) {
        const { projectId, projectKey, issueId, issueKey, headers, errorStrategy } = options ?? {};
        const queryParams = { projectId, projectKey, issueId, issueKey };
        const request = new Request('GET', constructUrl('/rest/api/2/mypermissions', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns all permissions that are present in the JIRA instance - Global, Project and the global ones added by plugins
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#-getAllPermissions
     * @param options Request options.
     */
    async getPermissions(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/permissions', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class PermissionSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Attribute = new PermissionSchemeAttributeGroup(implementation, this.getGlobalErrorStrategy);
        this.Grant = new PermissionSchemeGrantGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a list of all permission schemes.
     * By default only shortened beans are returned.
     * If you want to include permissions of all the schemes, then specify the permissions expand parameter.
     * Permissions will be included also if you specify any other expand parameter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemes
     * @param options Request options.
     */
    async getSchemes(options) {
        const { expand, headers, errorStrategy } = options ?? {};
        const queryParams = { expand };
        const request = new Request('GET', constructUrl('/rest/api/2/permissionscheme', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of all permission schemes.
     * By default only shortened beans are returned.
     * If you want to include permissions of all the schemes, then specify the permissions expand parameter.
     * Permissions will be included also if you specify any other expand parameter.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemes
     * @param options Request options.
     */
    async createScheme(options) {
        const { expand, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('POST', constructUrl('/rest/api/2/permissionscheme', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a permission scheme identified by the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { expand, schemeId, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a permission scheme identified by the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-deletePermissionScheme
     * @param options Request options.
     */
    async deleteScheme(options) {
        const { schemeId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Updates a permission scheme.
     * If the permissions list is present then it will be set in the permission scheme, which basically means it will overwrite any permission grants that existed in the permission scheme.
     * Sending an empty list will remove all permission grants from the permission scheme.
     * To update just the name and description, do not send permissions list at all.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-updatePermissionScheme
     * @param options Request options.
     */
    async updateScheme(options) {
        const { schemeId, expand, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('PUT', constructUrl(`/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class PermissionSchemeAttributeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns scheme attribute.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getSchemeAttribute
     * @param options Request options.
     */
    async getAttribute(options) {
        const { permissionSchemeId, attributeKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/permissionscheme/${encodeURIComponent(permissionSchemeId)}/attribute/${encodeURIComponent(attributeKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class PermissionSchemeGrantGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all permission grants of the given permission scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemeGrants
     * @param options Request options.
     */
    async getGrants(options) {
        const { schemeId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}/permission`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates a permission grant in a permission scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-createPermissionGrant
     * @param options Request options.
     */
    async createGrant(options) {
        const { schemeId, expand, body, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('POST', constructUrl(`/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}/permission`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a permission grant from a permission scheme.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-deletePermissionSchemeEntity
     * @param options Request options.
     */
    async deleteGrant(options) {
        const { schemeId, permissionId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}/permission/${encodeURIComponent(permissionId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns a permission grant identified by the given id.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#permissionscheme-getPermissionSchemeGrant
     * @param options Request options.
     */
    async getGrant(options) {
        const { schemeId, permissionId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/permissionscheme/${encodeURIComponent(schemeId)}/permission/${encodeURIComponent(permissionId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Component = new ComponentGroup(implementation, this.getGlobalErrorStrategy);
        this.Role = new ProjectRoleGroup(implementation, this.getGlobalErrorStrategy);
        this.Type = new ProjectTypeGroup(implementation, this.getGlobalErrorStrategy);
        this.Validation = new ProjectValidationGroup(implementation, this.getGlobalErrorStrategy);
        this.Version = new ProjectVersionGroup(implementation, this.getGlobalErrorStrategy);
        this.Issue = new ProjectIssueGroup(implementation, this.getGlobalErrorStrategy);
        this.Notification = new ProjectNotificationGroup(implementation, this.getGlobalErrorStrategy);
        this.Permission = new ProjectPermissionGroup(implementation, this.getGlobalErrorStrategy);
        this.Priority = new ProjectPriorityGroup(implementation, this.getGlobalErrorStrategy);
        this.Security = new ProjectSecurityGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns all projects which are visible for the currently logged in user. If no user is logged in, it returns the list of projects that are visible when using anonymous access.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getAllProjects
     * @param options Request options.
     */
    async getProjects(options) {
        const { expand, includeArchived, recent, headers, errorStrategy } = options ?? {};
        const queryParams = { expand, includeArchived, recent };
        const request = new Request('GET', constructUrl('/rest/api/2/project', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates a new project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-createProject
     * @param options Request options.
     */
    async createProject(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/project', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Contains a full representation of a project in JSON format.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProject
     * @param options Request options.
     */
    async getProject(options) {
        const { projectIdOrKey, headers, expand, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Contains a full representation of a project in JSON format.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProject
     * @param options Request options.
     */
    async updateProject(options) {
        const { projectIdOrKey, body, headers, expand, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('PUT', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-deleteProject
     * @param options Request options.
     */
    async deleteProject(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Archives a project. Only for DC.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-archiveProject
     * @param options Request options.
     */
    async archiveProject(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/archive`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Contains a full representation of a the specified project's components.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectComponents
     * @param options Request options.
     */
    async getComponents(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/components`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Contains a full representation of a the specified project's components.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectComponents
     * @param options Request options.
     */
    async restoreProject(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/restore`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Get all issue types with valid status values for a project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getAllStatuses
     * @param options Request options.
     */
    async getStatuses(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/statuses`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates the type of a project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-updateProjectType
     * @param options Request options.
     */
    async updateProjectType(options) {
        const { projectIdOrKey, newProjectTypeKey, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/type/${encodeURIComponent(newProjectTypeKey)}`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns all versions for the specified project. Results are paginated.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectVersionsPaginated
     * @param options Request options.
     */
    async getVersionsPaginated(options) {
        const { projectIdOrKey, headers, expand, maxResults, orderBy, startAt, errorStrategy } = options;
        const queryParams = { expand, maxResults, orderBy, startAt };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/version`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Contains a full representation of a the specified project's versions.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project-getProjectVersions
     * @param options Request options.
     */
    async getVersions(options) {
        const { projectIdOrKey, headers, expand, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/versions`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ComponentGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Create a component via POST.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-createComponent
     * @param options Request options.
     */
    async createComponent(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/component', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a project component.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-getComponent
     * @param options Request options.
     */
    async getComponent(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/component/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Modify a component via PUT. If leadUserName is an empty string ("") the component lead will be removed.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-updateComponent
     * @param options Request options.
     */
    async updateComponent(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/component/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete a project component.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-delete
     * @param options Request options.
     */
    async deleteComponent(options) {
        const { id, moveIssuesTo, headers, errorStrategy } = options;
        const queryParams = { moveIssuesTo };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/component/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns counts of issues related to this component.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#component-getComponentRelatedIssues
     * @param options Request options.
     */
    async getRelatedIssueCount(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/component/${encodeURIComponent(id)}/relatedIssueCounts`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectRoleGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Actor = new RoleActorGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns all roles in the given project Id or key, with links to full details on each role.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-getProjectRoles
     * @param options Request options.
     */
    async getRoles(options) {
        const { projectIdOrKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/role`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the details for a given project role in a project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-getProjectRole
     * @param options Request options.
     */
    async getRole(options) {
        const { projectIdOrKey, id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/role/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RoleActorGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Updates a project role to include the specified actors (users or groups).
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-setActors
     * @param options Request options.
     */
    async setActors(options) {
        const { projectIdOrKey, id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/role/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds an actor (user or group) to a project role.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-addActorUsers
     * @param options Request options.
     */
    async addUsers(options) {
        const { projectIdOrKey, id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/role/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes actors (users or groups) from a project role.
     * Delete a user from the role: /rest/api/2/project/{projectIdOrKey}/role/{roleId}?user={username}
     * Delete a group from the role: /rest/api/2/project/{projectIdOrKey}/role/{roleId}?group={groupname}
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/{projectIdOrKey}/role-deleteActor
     * @param options Request options.
     */
    async deleteActor(options) {
        const { projectIdOrKey, id, headers, group, user, errorStrategy } = options;
        const queryParams = { group, user };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectIdOrKey)}/role/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class ProjectSecurityGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all security levels for the project that the current logged in user has access to.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/securitylevel-getSecurityLevelsForProject
     * @param options Request options.
     */
    async getLevels(options) {
        const { projectKeyOrId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectKeyOrId)}/securitylevel`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectTypeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns all the project types defined on the JIRA instance, not taking into account whether the license to use those project types is valid or not.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getAllProjectTypes
     * @param options Request options.
     */
    async getTypes(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/project/type', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the project type with the given key.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getProjectTypeByKey
     * @param options Request options.
     */
    async getType(options) {
        const { projectTypeKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/type/${encodeURIComponent(projectTypeKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the project type with the given key, if it is accessible to the logged in user. This takes into account whether the user is licensed on the Application that defines the project type.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#project/type-getAccessibleProjectTypeByKey
     * @param options Request options.
     */
    async getAccessibleType(options) {
        const { projectTypeKey, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/type/${encodeURIComponent(projectTypeKey)}/accessible`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectValidationGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Validates a project key.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#projectvalidate-getProject
     * @param options Request options.
     */
    async validateProjectKey(options) {
        const { key, headers, errorStrategy } = options;
        const queryParams = { key };
        const request = new Request('GET', constructUrl('/rest/api/2/projectvalidate/key', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectVersionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Count = new VersionCountGroup(implementation, this.getGlobalErrorStrategy);
        this.Remote = new ProjectVersionRemoteGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Create a version via POST.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-createVersion
     * @param options Request options.
     */
    async createVersion(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/version', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Modify a version's sequence within a project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-moveVersion
     * @param options Request options.
     */
    async moveVersion(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/version/${encodeURIComponent(id)}/move`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a project version.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersion
     * @param options Request options.
     */
    async getVersion(options) {
        const { id, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/version/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Modify a version via PUT. Any fields present in the PUT will override existing values. As a convenience, if a field is not present, it is silently ignored.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-updateVersion
     * @param options Request options.
     */
    async updateVersion(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/version/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete a project version.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-delete
     * @param options Request options.
     */
    async deleteVersion(options) {
        const { id, headers, moveAffectedIssuesTo, moveFixIssuesTo, errorStrategy } = options;
        const queryParams = { moveAffectedIssuesTo, moveFixIssuesTo };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/version/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Merge versions.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-merge
     * @param options Request options.
     */
    async mergeVersion(options) {
        const { id, moveIssuesTo, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/version/${encodeURIComponent(id)}/mergeto/${encodeURIComponent(moveIssuesTo)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Delete a project version.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-delete
     * @param options Request options.
     */
    async removeAndReplaceVersion(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/version/${encodeURIComponent(id)}/removeAndSwap`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class VersionCountGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a bean containing the number of fixed in and affected issues for the given version.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersionRelatedIssues
     * @param options Request options.
     */
    async getVersionRelatedIssueCount(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/version/${encodeURIComponent(id)}/relatedIssueCounts`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the number of unresolved issues for the given version
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#version-getVersionUnresolvedIssues
     * @param options Request options.
     */
    async getVersionUnresolvedIssueCount(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/version/${encodeURIComponent(id)}/unresolvedIssueCount`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectVersionRemoteGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the remote version links associated with the given version ID.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-getRemoteVersionLinksByVersionId
     * @param options Request options.
     */
    async getLinksById(options) {
        const { versionId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Create a remote version link via POST.
     * The link's global ID will be taken from the JSON payload if provided; otherwise, it will be generated.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-createOrUpdateRemoteVersionLink
     * @param options Request options.
     */
    async createOrUpdateLinkById(options) {
        const { versionId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Delete all remote version links for a given version ID.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-deleteRemoteVersionLinksByVersionId
     * @param options Request options.
     */
    async deleteLinksById(options) {
        const { versionId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * A REST sub-resource representing a remote version link.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-getRemoteVersionLink
     * @param options Request options.
     */
    async getLink(options) {
        const { versionId, globalId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink/${encodeURIComponent(globalId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Create a remote version link via POST.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-createOrUpdateRemoteVersionLink
     * @param options Request options.
     */
    async createOrUpdateLink(options) {
        const { versionId, globalId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink/${encodeURIComponent(globalId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Delete a specific remote version link with the given version ID and global ID.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-deleteRemoteVersionLink
     * @param options Request options.
     */
    async deleteLink(options) {
        const { versionId, globalId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/version/${encodeURIComponent(versionId)}/remotelink/${encodeURIComponent(globalId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns the remote version links for a given global ID.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#version-getRemoteVersionLinks
     * @param options Request options.
     */
    async getLinks(options) {
        const { globalId, headers, errorStrategy } = options;
        const queryParams = { globalId };
        const request = new Request('GET', constructUrl('/rest/api/2/version/remotelink', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectIssueGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Security = new ProjectIssueSecurityGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ProjectIssueSecurityGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new ProjectIssueSecuritySchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ProjectIssueSecuritySchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns the issue security scheme for project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/issuesecuritylevelscheme-getIssueSecurityScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { projectKeyOrId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/project/${encodeURIComponent(projectKeyOrId)}/issuesecuritylevelscheme`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectNotificationGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new ProjectNotificationSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ProjectNotificationSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets a notification scheme associated with the project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/notificationscheme-getNotificationScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { projectKeyOrId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectKeyOrId)}/notificationscheme`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectPermissionGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new ProjectPermissionSchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ProjectPermissionSchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Resource for associating permission schemes and projects.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/permissionscheme-getAssignedPermissionScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { projectKeyOrId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectKeyOrId)}/permissionscheme`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectPriorityGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Scheme = new ProjectPrioritySchemeGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class ProjectPrioritySchemeGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets a full representation of a priority scheme in JSON format used by specified project.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#project/{projectKeyOrId}/priorityscheme-getAssignedPriorityScheme
     * @param options Request options.
     */
    async getScheme(options) {
        const { projectKeyOrId, expand, headers, errorStrategy } = options;
        const queryParams = { expand };
        const request = new Request('GET', constructUrl(`/rest/api/2/project/${encodeURIComponent(projectKeyOrId)}/priorityscheme`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ScreenGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Tab = new ScreenTabGroup(implementation, this.getGlobalErrorStrategy);
        this.Field = new ScreenFieldGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Gets available fields for screen. i.e ones that haven't already been added.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getFieldsToAdd
     * @param options Request options.
     */
    async getFieldsToAdd(options) {
        const { screenId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/screens/${encodeURIComponent(screenId)}/availableFields`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Gets available field screens.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllScreens
     * @param options Request options.
     */
    async getScreens(options) {
        const { startAt, maxResults, expand, search, headers, errorStrategy } = options ?? {};
        const queryParams = { startAt, maxResults, expand, search };
        const request = new Request('GET', constructUrl('/rest/api/2/screens', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ScreenTabGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Creates tab for given screen.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addTab
     * @param options Request options.
     */
    async addTab(options) {
        const { screenId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of all tabs for the given screen.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllTabs
     * @param options Request options.
     */
    async getTabs(options) {
        const { screenId, projectKey, headers, errorStrategy } = options;
        const queryParams = { projectKey };
        const request = new Request('GET', constructUrl(`/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Renames tab on given screen.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-renameTab
     * @param options Request options.
     */
    async renameTab(options) {
        const { screenId, tabId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes tab to give screen.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-deleteTab
     * @param options Request options.
     */
    async deleteTab(options) {
        const { screenId, tabId, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Moves tab position.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-moveTab
     * @param options Request options.
     */
    async moveTab(options) {
        const { screenId, tabId, pos, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}/move/${encodeURIComponent(pos)}`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class ScreenFieldGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets all fields for a given tab.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-getAllFields
     * @param options Request options.
     */
    async getFields(options) {
        const { screenId, tabId, projectKey, headers, errorStrategy } = options;
        const queryParams = { projectKey };
        const request = new Request('GET', constructUrl(`/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}/fields`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
    * Adds field to the given tab.
    * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addField
    * @param options Request options.
    */
    async addField(options) {
        const { screenId, tabId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}/fields`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes field from given tab.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-removeField
     * @param options Request options.
     */
    async removeField(options) {
        const { screenId, tabId, id, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}/fields/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Moves field on the given tab.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-moveField
     * @param options Request options.
     */
    async moveField(options) {
        const { screenId, tabId, id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/screens/${encodeURIComponent(screenId)}/tabs/${encodeURIComponent(tabId)}/fields/${encodeURIComponent(id)}/move`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Adds field or custom field to the default tab.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#screens-addFieldToDefaultScreen
     * @param options Request options.
     */
    async addToDefaultScreen(options) {
        const { fieldId, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/screens/addToDefault/${encodeURIComponent(fieldId)}`, constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class UserGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Search = new UserSearchGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Returns a user.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-getUser
     * @param options Request options.
     */
    async getUser(options) {
        const { key, username, headers, errorStrategy } = options ?? {};
        const queryParams = { key, username };
        const request = new Request('GET', constructUrl('/rest/api/2/user', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Modify user. The "value" fields present will override the existing value. Fields skipped in request will not be changed.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-updateUser
     * @param options Request options.
     */
    async updateUser(options) {
        const { key, username, body, headers, errorStrategy } = options;
        const queryParams = { key, username };
        const request = new Request('PUT', constructUrl('/rest/api/2/user', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Create user. By default created user will not be notified with email. If password field is not set then password will be randomly generated.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-createUser
     * @param options Request options.
     */
    async createUser(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/user', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes user and its references (like project roles associations, watches, history).
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-removeUser
     * @param options Request options.
     */
    async removeUser(options) {
        const { key, username, headers, errorStrategy } = options;
        const queryParams = { key, username };
        const request = new Request('DELETE', constructUrl('/rest/api/2/user', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Add user to given application. Admin permission will be required to perform this operation.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-addUserToApplication
     * @param options Request options.
     */
    async addUserToApplication(options) {
        const { applicationKey, username, headers, errorStrategy } = options;
        const queryParams = { applicationKey, username };
        const request = new Request('POST', constructUrl('/rest/api/2/user/application', queryParams), constructHeaders(null, headers), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Remove user from given application. Admin permission will be required to perform this operation.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-removeUserFromApplication
     * @param options Request options.
     */
    async removeUserFromApplication(options) {
        const { applicationKey, username, headers, errorStrategy } = options;
        const queryParams = { applicationKey, username };
        const request = new Request('DELETE', constructUrl('/rest/api/2/user/application', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Modify user password.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-changeUserPassword
     * @param options Request options.
     */
    async changeUserPassword(options) {
        const { key, username, body, headers, errorStrategy } = options;
        const queryParams = { key, username };
        const request = new Request('PUT', constructUrl('/rest/api/2/user/password', queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class UserSearchGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Returns a list of users that match the search string. This resource cannot be accessed anonymously. Please note that this resource should be called with an issue key when a list of assignable users is retrieved for editing. For create only a project key should be supplied. The list of assignable users may be incorrect if it's called with the project key for editing.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findAssignableUsers
     * @param options Request options.
     */
    async findAssignableUsers(options) {
        const { username, startAt, maxResults, actionDescriptorId, issueKey, project, headers, errorStrategy } = options ?? {};
        const queryParams = { username, startAt, maxResults, actionDescriptorId, issueKey, project };
        const request = new Request('GET', constructUrl('/rest/api/2/user/assignable/search', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of active users that match the search string and have all specified permissions for the project or issue.
     * This resource can be accessed by users with ADMINISTER_PROJECT permission for the project or global ADMIN or SYSADMIN rights.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsersWithAllPermissions
     * @param options Request options.
     */
    async findUsersWithAllPermissions(options) {
        const { username, startAt, maxResults, issueKey, permissions, projectKey, headers, errorStrategy } = options;
        const queryParams = { username, startAt, maxResults, issueKey, permissions, projectKey };
        const request = new Request('GET', constructUrl('/rest/api/2/user/permission/search', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of users that match the search string. This resource cannot be accessed anonymously.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsers
     * @param options Request options.
     */
    async findUsers(options) {
        const { username, startAt, maxResults, includeActive, includeInactive, headers, errorStrategy } = options;
        const queryParams = { username, startAt, maxResults, includeActive, includeInactive };
        const request = new Request('GET', constructUrl('/rest/api/2/user/search', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns a list of active users that match the search string. This resource cannot be accessed anonymously and requires the Browse Users global permission. Given an issue key this resource will provide a list of users that match the search string and have the browse issue permission for the issue provided.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#user-findUsersWithBrowsePermission
     * @param options Request options.
     */
    async findUsersWithBrowsePermission(options) {
        const { username, startAt, maxResults, projectKey, issueKey, headers, errorStrategy } = options;
        const queryParams = { username, startAt, maxResults, projectKey, issueKey };
        const request = new Request('GET', constructUrl('/rest/api/2/user/viewissue/search', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RoleGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Project = new RoleProjectGroup(implementation, this.getGlobalErrorStrategy);
    }
}
class RoleProjectGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Actor = new RoleProjectActorGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Get all the ProjectRoles available in JIRA. Currently this list is global.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRoles
     * @param options Request options.
     */
    async getRoles(options) {
        const { headers, errorStrategy } = options ?? {};
        const request = new Request('GET', '/rest/api/2/role', constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates a new ProjectRole to be available in Jira. The created role does not have any default actors assigned.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-createProjectRole
     * @param options Request options.
     */
    async createRole(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/2/role', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Get a specific ProjectRole available in JIRA.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRolesById
     * @param options Request options.
     */
    async getRole(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/role/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Partially updates a roles name or description.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-partialUpdateProjectRole
     * @param options Request options.
     */
    async updateRolePartially(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/role/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Fully updates a roles. Both name and description must be given.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.0/#role-getProjectRolesById
     * @param options Request options.
     */
    async updateRoleFully(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/2/role/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a role. May return 403 in the future.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-deleteProjectRole
     * @param options Request options.
     */
    async deleteRole(options) {
        const { id, swap, headers, errorStrategy } = options;
        const queryParams = { swap };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/role/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
class RoleProjectActorGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets default actors for the given role.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-getProjectRoleActorsForRole
     * @param options Request options.
     */
    async getActors(options) {
        const { id, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/2/role/${encodeURIComponent(id)}/actors`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Adds default actors to the given role. The request data should contain a list of usernames or a list of groups to add.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-addProjectRoleActorsToRole
     * @param options Request options.
     */
    async addActors(options) {
        const { id, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/2/role/${encodeURIComponent(id)}/actors`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Removes default actor from the given role.
     * Documentation: https://docs.atlassian.com/software/jira/docs/api/REST/8.20.2/#role-deleteProjectRoleActorsFromRole
     * @param options Request options.
     */
    async deleteActors(options) {
        const { id, user, group, headers, errorStrategy } = options;
        const queryParams = { user, group };
        const request = new Request('DELETE', constructUrl(`/rest/api/2/role/${encodeURIComponent(id)}/actors`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
